Package: qshap
Type: Package
Title: Fast Calculation of Feature Contributions in Boosting Trees
Version: 1.0.1
Date: 2026-05-03
Authors@R: c(
    person("Steven", "He", role = "aut", email = "hess1@uci.edu"),
    person("Zhongli", "Jiang", role = c("aut", "cre"),
           email = "zhongli.jiang.stats@gmail.com"),
    person("Dabao", "Zhang", role = "aut", email = "zdb969@hs.uci.edu"))
Description: Computes feature-specific R-squared (R2) contributions for
    boosting tree models using a Shapley-value-based decomposition of the
    total R-squared in polynomial time. Supports models fitted with
    'XGBoost', 'LightGBM', and 'CatBoost', with optimized backend-specific
    implementations and cached tree summaries suitable for large-scale
    problems. Multiple
    visualization tools are included for interpreting and communicating
    feature contributions. The methodology is described in Jiang, Zhang,
    and Zhang (2025) <doi:10.48550/arXiv.2407.03515>. Optional 'CatBoost'
    support uses the R package 'catboost', which is not distributed on CRAN;
    installation instructions and released binaries are provided by the
    CatBoost project at <https://catboost.ai/docs/en/concepts/r-installation>.
License: GPL (>= 2)
URL: https://github.com/catstats/Q-SHAP_R
BugReports: https://github.com/catstats/Q-SHAP_R/issues
Imports: Rcpp (>= 1.0.14), xgboost (>= 3.1.3.1), parallel, lightgbm,
        viridisLite, ggplot2, jsonlite, methods, progress
Suggests: shiny
LinkingTo: Rcpp, RcppEigen
RoxygenNote: 7.3.2
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-05-03 22:16:54 UTC; jiangzhongli
Author: Steven He [aut],
  Zhongli Jiang [aut, cre],
  Dabao Zhang [aut]
Maintainer: Zhongli Jiang <zhongli.jiang.stats@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-04 07:00:17 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-04 22:53:48 UTC; unix
Archs: qshap.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,qshap_result)
S3method(gazer,catboost.Model)
S3method(gazer,default)
S3method(gazer,lgb.Booster)
S3method(gazer,xgb.Booster)
S3method(plot,qshap_rsq)
S3method(print,qshap_result)
S3method(print,qshap_rsq)
S3method(print,qshap_tree_explainer)
S3method(print,simple_tree)
S3method(print,tree_summary)
S3method(qshap_loss,qshap_tree_explainer)
S3method(summary,qshap_result)
S3method(summary,qshap_rsq)
S3method(summary,qshap_tree_explainer)
export(gazer)
export(loss)
export(qshap)
export(qshap_result)
export(rsq)
import(Rcpp)
import(ggplot2)
import(lightgbm)
importFrom(methods,new)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,clusterExport)
importFrom(parallel,detectCores)
importFrom(parallel,makeCluster)
importFrom(parallel,parLapply)
importFrom(parallel,stopCluster)
importFrom(stats,predict)
importFrom(xgboost,xgb.model.dt.tree)
useDynLib(qshap, .registration = TRUE)

# Q-SHAP: Feature-Specific $R^2$ Values for Tree Ensembles

[![PyPI](https://img.shields.io/pypi/v/qshap)](https://pypi.org/project/qshap/)
![Downloads](https://img.shields.io/pypi/dm/qshap)

This package is used to compute feature-specific $R^2$ values, following Shapley decomposition of the total $R^2$, for tree ensembles in polynomial time based on the [paper](https://arxiv.org/abs/2407.03515).

This version only takes outputs from **XGBoost**, **scikit-learn Decision Tree**, and **scikit-learn GBDT**. We are working to update it for random forests in the next version. Please check **Q-SHAP Tutorial.ipynb** for more details using Q-SHAP.

## Installation

`qshap` can be installed through PyPI:

<pre>
pip install qshap
</pre>

## Quick Start

```python
# Import necessary libraries
from sklearn.datasets import fetch_california_housing
from qshap import gazer, vis
import xgboost as xgb
import numpy as np

# Load the California Housing dataset and fit a XGBoost regressor
housing = fetch_california_housing()
x, y, feature_names = housing.data, housing.target, housing.feature_names
model = xgb.XGBRegressor(max_depth=2, n_estimators=50, random_state=42).fit(x, y)

# Obtain feature-specific R^2 using qshap, using 5% of the data (around 1000)
gazer_rsq = gazer(model)
phi_rsq = gazer.rsq(gazer_rsq, x, y, nfrac = 0.05, random_state=42)

# Visualize top values of feature-specific R^2
vis.rsq(phi_rsq, label=np.array(feature_names), rotation=30, save_name="cal_housing", color_map_name="Pastel2")
```

<p align="center">
  <img width="500" src="./figs/cal_housing.png" />
</p>

## Citation

```bibtex
@article{jiang2024feature,
  title={Feature-Specific Coefficients of Determination in Tree Ensembles},
  author={Jiang, Zhongli and Zhang, Dabao and Zhang, Min},
  journal={arXiv preprint arXiv:2407.03515},
  year={2024}
}
```

## References

- Jiang, Z., Zhang, D., & Zhang, M. (2024). "Feature-specific coefficients of determination in tree ensembles." arXiv preprint arXiv:2407.03515.
- Lundberg, Scott M., et al. "From local explanations to global understanding with explainable AI for trees." Nature Machine Intelligence 2.1 (2020): 56-67.
- Karczmarz, Adam, et al. "Improved feature importance computation for tree models based on the Banzhaf value." Uncertainty in Artificial Intelligence. PMLR, 2022.
- Bifet, Albert, Jesse Read, and Chao Xu. "Linear tree shap." Advances in Neural Information Processing Systems 35 (2022): 25818-25828.
- Chen, Tianqi, and Carlos Guestrin. "Xgboost: A scalable tree boosting system." Proceedings of the 22nd ACM SIGKDD International Conference on Knowledge Discovery and Data Mining. 2016.

## Container Images

We provide pre-built images, available for both Docker and Singularity, with all necessary packages for Q-SHAP in Python 3.11:

- **Docker:**  
  You can pull the Docker image using the following command:
  ```sh
  docker pull catstat/xai
  ```
- **Singularity:**  
  You can pull the Docker image using the following command:
  ```sh
  singularity pull docker://catstat/xai:0.0
  ```

## Task List

- [ ] Task 1: Lightgbm version
- [ ] Task 2: Catboost version

#' @import Rcpp
#' @importFrom methods new
#' @importFrom parallel makeCluster stopCluster detectCores
#' @importFrom parallel clusterEvalQ clusterExport parLapply
#' @useDynLib qshap, .registration = TRUE
NULL

#' Create a QSHAP Tree Explainer
#' 
#' Creates an explainer object for computing feature-specific Shapley values
#' from a trained tree ensemble model. Supports XGBoost, LightGBM, and
#' CatBoost models.
#' 
#' @param model A model object of class \code{xgboost} or \code{xgb.Booster} from \pkg{xgboost}, class \code{lgb.Booster} from \pkg{lightgbm}, or class \code{catboost.Model} from \pkg{catboost}
#' @param max_depth Maximum depth of trees, extracted from \code{model} by default.
#' @param base_score Base score for predictions, extracted from \code{model} by default.
#' @param ... Additional arguments, for future use
#' 
#' @return A class of \code{qshap_tree_explainer} object containing the model information and
#'   preprocessed tree structures for fast Shapley value computation
#'   
#' @examples
#' library(xgboost)
#' set.seed(42)
#' n <- 100
#' p <- 100
#' X <- matrix(rnorm(n * p), nrow = n, ncol = p)
#' y <- X[, 1] - X[, 2] + rnorm(n, sd = 0.2)
#' model <- xgboost(X, y, nrounds = 15, max_depth = 2, verbose = 0)
#' explainer <- gazer(model)
#'
#' @export
gazer <- function(model, max_depth = NULL, base_score = NULL, ...) {
  UseMethod("gazer")
}

cache_tree_summaries <- function(explainer) {
  explainer$tree_summaries <- lapply(explainer$trees, summarize_tree)
  explainer
}

#' @export
gazer.xgb.Booster <- function(model, ...) {


  tmp <- tempfile(fileext = ".json")
  xgboost::xgb.save(model, tmp)
  model_json <- jsonlite::fromJSON(tmp, simplifyVector = FALSE)
  unlink(tmp)

  # For version 3.1.3.1 and later, we can get max_depth from attributes
  max_depth <- if (!is.null(attributes(model)$params$max_depth)) attributes(model)$params$max_depth else 6

  # Extract base_score - handle various JSON formats
  base_score_raw <- model_json$learner$learner_model_param$base_score
  if (is.character(base_score_raw)) {
    # Handle string format like "[-7.70459E-2]" or just a number string
    base_score_raw <- gsub("\\[|\\]", "", base_score_raw)  # Remove brackets
    base_score <- as.numeric(base_score_raw)
  } else if (is.list(base_score_raw)) {
    base_score <- as.numeric(base_score_raw[[1]])
  } else {
    base_score <- as.numeric(base_score_raw)
  }
  # Fallback to mean of predictions if still NA
  if (is.na(base_score)) {
    warning("Could not extract base_score from model, using 0.5 as default")
    base_score <- 0.5
  }

  # eta: try params first, else JSON
  # eta <- model$params$eta
  # if (is.null(eta)) {
  #   # common JSON locations depending on xgboost build
  #   eta <- model_json$learner$gradient_booster$gbtree_train_param$learning_rate
  #   if (is.null(eta)) eta <- model_json$learner$gradient_booster$gbtree_train_param$eta
  # }
  # if (is.null(eta)) eta <- 0.3
  # eta <- as.numeric(eta)

  xgb_trees <- xgb_formatter(model_json, max_depth)

  explainer <- new_qshap_tree_explainer(
    model = model,
    model_type = "xgboost",
    max_depth = max_depth,
    base_score = base_score,
    trees = xgb_trees,
    store_v_invc = store_complex_v_invc(max_depth * 2),
    store_z = store_complex_root(max_depth * 2)
  )

  explainer <- cache_tree_summaries(explainer)
  
  validate_qshap_tree_explainer(explainer)
  explainer
}

#' @export
gazer.lgb.Booster <- function(model, max_depth = NULL, ...) {
  # Get max_depth from model parameters or use default
  max_depth <- if (!is.null(model$params$max_depth)) model$params$max_depth else 31
  
  # Format LightGBM trees
  lgb_trees <- lgb_formatter(model, max_depth)

  explainer <- new_qshap_tree_explainer(
    model = model,
    model_type = "lightgbm",
    max_depth = max_depth,
    base_score = NULL,
    trees = lgb_trees,
    store_v_invc = store_complex_v_invc(max_depth * 2),
    store_z = store_complex_root(max_depth * 2)
  )

  explainer <- cache_tree_summaries(explainer)
  
  validate_qshap_tree_explainer(explainer)
  explainer
}

#' @export
gazer.catboost.Model <- function(model, max_depth = NULL, ...) {
  # Format CatBoost trees (returns list with metadata attributes)
  cb_trees <- catboost_formatter(model, max_depth)

  # Extract metadata set by formatter
  bias <- attr(cb_trees, "bias")
  if (is.null(bias)) bias <- 0.0
  actual_max_depth <- attr(cb_trees, "max_depth")
  if (is.null(actual_max_depth)) actual_max_depth <- 6L

  if (is.null(max_depth)) max_depth <- actual_max_depth

  explainer <- new_qshap_tree_explainer(
    model = model,
    model_type = "catboost",
    max_depth = max_depth,
    base_score = bias,
    trees = cb_trees,
    store_v_invc = store_complex_v_invc(max_depth * 2),
    store_z = store_complex_root(max_depth * 2)
  )

  explainer <- cache_tree_summaries(explainer)

  validate_qshap_tree_explainer(explainer)
  explainer
}

# #' @export
# gazer.gbm <- function(model, max_depth = NULL, ...) {
# }

#' @export
gazer.default <- function(model, ...) {
  stop(sprintf("gazer not implemented for class %s", class(model)[1]))
}

#' Calculate Q-SHAP Loss Contributions
#' 
#' Computes the feature-specific loss contributions using Q-SHAP decomposition.
#' This is an internal function typically called by \code{rsq()}.
#' 
#' @param explainer A qshap_tree_explainer object created by \code{gazer()}
#' @param x Feature matrix or data frame
#' @param y Response vector
#' @param y_mean_ori Optional pre-computed mean of y (for efficiency)
#' 
#' @return A matrix of loss contributions with dimensions (n_samples, n_features)
#' 
#' @keywords internal
qshap_loss <- function(explainer, x, y, y_mean_ori = NULL) {
  UseMethod("qshap_loss")
}

#' @export
qshap_loss.qshap_tree_explainer <- function(explainer, x, y, y_mean_ori = NULL) {
  switch(explainer$model_type,
    "xgboost" = qshap_loss_xgboost(explainer, x, y, y_mean_ori),
    "lightgbm" = qshap_loss_lightgbm(explainer, x, y, y_mean_ori),
    "catboost" = qshap_loss_catboost(explainer, x, y, y_mean_ori),
    # "gbm" = qshap_loss_gbm(explainer, x, y, y_mean_ori, progress_bar),
    stop("Unknown model type: ", explainer$model_type)
  )
}

 #' Calculate Feature-Specific R-Squared Values
 #' 
 #' Computes feature-specific R-squared values using Q-SHAP decomposition.
 #' Supports parallel processing and sampling for large datasets.
 #' 
 #' @param explainer A qshap_tree_explainer object created by \code{gazer()}
 #' @param x Feature matrix or data frame with n samples and p features
 #' @param y Response vector of length n
 #' @param local Logical; if TRUE, returns both R-squared values and loss matrix
 #' @param sd_out Logical; if TRUE, returns standard deviations of R-squared estimates
 #' @param ci_out Logical; if TRUE, returns Wald-style confidence intervals for each feature's R-squared (normal approximation using sd_rsq)
 #' @param level Confidence level for the intervals (default 0.95)
 #' @param nsample Optional integer; number of samples to use (random subsample if less than nrow(x))
 #' @param nfrac Optional numeric in (0,1); fraction of samples to use (alternative to nsample)
 #' @param random_state Integer seed for reproducible sampling
 #' @param ncore Number of cores for parallel processing. Use -1 for all available cores, 
 #'   or a positive integer. Default is 1 (no parallelization)
 #' 
 #' @return If \code{local=FALSE} (default), returns a numeric vector of length p 
 #'   containing feature-specific R-squared values. If \code{local=TRUE}, returns 
 #'   a list with components \code{rsq} (the R-squared vector) and \code{loss} 
 #'   (an n x p matrix of loss contributions). When \code{ci_out=TRUE}, the returned list
 #'   also contains \code{ci_lower} and \code{ci_upper} vectors representing Wald-style confidence intervals.
 #'   
 #' @examples
 #' library(xgboost)
 #' set.seed(42)
 #' n <- 100
 #' p <- 100
 #' X <- matrix(rnorm(n * p), nrow = n, ncol = p)
 #' y <- X[, 1] - X[, 2] + rnorm(n, sd = 0.2)
 #' model <- xgboost(X, y, nrounds = 15, max_depth = 2, verbose = 0)
 #' explainer <- gazer(model)
 #' phi_rsq <- qshap(explainer, X, y)
 #' print(phi_rsq)
 #'
 #' @keywords internal
qshap_rsq <- function(explainer, x, y, local = FALSE, nsample = NULL, sd_out = TRUE,
                      ci_out = TRUE, level = 0.95,
                      nfrac = NULL, random_state = 42,
                      ncore = 1L) {
  # Sampling logic
  if (!is.null(nsample)) {
    if (nsample <= 0 || nsample >= nrow(x)) {
      stop("nsample must be > 0 and < total number of samples")
    }
    set.seed(random_state)
    sample_idx <- sample(nrow(x), nsample, replace = FALSE)
    x <- x[sample_idx, , drop = FALSE]
    y <- y[sample_idx]
  } else if (!is.null(nfrac)) {
    if (nfrac <= 0 || nfrac >= 1) {
      stop("nfrac must be between 0 and 1")
    }
    set.seed(random_state)
    nsample <- floor(nrow(x) * nfrac)
    sample_idx <- sample(nrow(x), nsample, replace = FALSE)
    x <- x[sample_idx, , drop = FALSE]
    y <- y[sample_idx]
  }

  # CI implies we need sd estimates
  if (isTRUE(ci_out)) sd_out <- TRUE
  if (is.null(level) || length(level) != 1 || is.na(level) || level <= 0 || level >= 1) {
    stop("level must be a single number in (0, 1)")
  }

  ci_from_sd <- function(rsq, sd_rsq, level) {
    z <- stats::qnorm(1 - (1 - level) / 2)
    list(
      ci_lower = rsq - z * sd_rsq,
      ci_upper = rsq + z * sd_rsq
    )
  }

  y_mean_ori <- mean(y)
  sst <- sum((y - y_mean_ori)^2)

  # Parallel if number of samples is large enough
  # Normalize ncore
  if (is.null(ncore) || length(ncore) != 1 || is.na(ncore)) ncore <- 1L
  ncore <- as.integer(ncore)
  max_core <- parallel::detectCores(logical = TRUE)
  if (is.na(max_core) || max_core < 1) max_core <- 1L
  if (ncore == -1L) ncore <- max_core
  ncore <- max(1L, min(max_core, ncore))

  n <- nrow(x)

  # Calculate loss (serial)
  if (ncore == 1L || n <= 1L) {
    loss <- qshap_loss(explainer, x, y, y_mean_ori)
    rsq <- -colSums(loss) / sst

     if (sd_out) {
      loss_mean <- colMeans(loss)
      if (n > 1L) {
        loss_sumsq <- colSums(loss * loss)
        loss_var <- pmax((loss_sumsq - n * loss_mean^2) / (n - 1), 0)
        sd_rsq <- sqrt(n * loss_var) / sst
      } else {
        sd_rsq <- rep(NA_real_, ncol(loss))
      }
    } else {
      sd_rsq <- NULL
    }
    if (ci_out && !is.null(sd_rsq)) {
      ci <- ci_from_sd(rsq, sd_rsq, level)
    } else {
      ci <- NULL
    }

    if (local) {
      out <- list(rsq = rsq, loss = loss, sd_rsq = sd_rsq)
    } else {
      out <- list(rsq = rsq, sd_rsq = sd_rsq)
    }
    if (!is.null(ci)) {
      out$ci_lower <- ci$ci_lower
      out$ci_upper <- ci$ci_upper
      out$level <- level
    }
    class(out) <- c("qshap_rsq", "list")
    return(out)
  }

  # Divide indices into ncore chunks (preserve order)
  idx_chunks <- split(seq_len(n), cut(seq_len(n), breaks = ncore, labels = FALSE))

  # Parallel compute using PSOCK cluster (CRAN/Windows friendly)
  cl <- parallel::makeCluster(ncore)
  on.exit(parallel::stopCluster(cl), add = TRUE)

  # Ensure package namespace is available on workers
  parallel::clusterEvalQ(cl, suppressPackageStartupMessages(library(qshap)))

  # Export needed data once (avoid resending for every task)
  parallel::clusterExport(
    cl,
    varlist = c("x", "y", "explainer", "y_mean_ori", "idx_chunks", "local"),
    envir = environment()
  )

  worker <- function(idx) {
  lc <- qshap::loss(explainer, x[idx, , drop = FALSE], y[idx], y_mean_ori)

  if (local) {
    # keep full chunk loss matrix
    return(lc)
  } else {
    # return sufficient statistics only
    return(list(
      sum   = colSums(lc),
      sumsq = if (sd_out) colSums(lc^2) else NULL,
      n     = length(idx)
    ))
  }
  }

  results <- parallel::parLapply(cl, idx_chunks, worker)

if (local) {
  # Combine full loss matrix
  loss <- do.call(rbind, results)
  n_all <- nrow(loss)

  # point estimate
  rsq <- -colSums(loss) / sst

  if (sd_out) {
      loss_mean <- colMeans(loss)
      if (n_all > 1L) {
        loss_sumsq <- colSums(loss * loss)
        loss_var <- pmax((loss_sumsq - n_all * loss_mean^2) / (n_all - 1), 0)
        sd_rsq <- sqrt(n_all * loss_var) / sst
      } else {
        sd_rsq <- rep(NA_real_, ncol(loss))
      }
    } else {
      sd_rsq <- NULL
    }
  if (ci_out && !is.null(sd_rsq)) {
    ci <- ci_from_sd(rsq, sd_rsq, level)
  } else {
    ci <- NULL
  }
  out <- list(rsq = rsq, loss = loss, sd_rsq = sd_rsq)
  if (!is.null(ci)) {
    out$ci_lower <- ci$ci_lower
    out$ci_upper <- ci$ci_upper
    out$level <- level
  }
  class(out) <- c("qshap_rsq", "list")
  return(out)

} else {
  # Combine sufficient statistics
  sum_all <- Reduce(`+`, lapply(results, `[[`, "sum"))
  n_all <- sum(vapply(results, `[[`, numeric(1), "n"))

  # point estimate
  rsq <- -sum_all / sst

  if (sd_out) {
      if (n_all > 1L) {
        sumsq_all <- Reduce(`+`, lapply(results, `[[`, "sumsq"))
        loss_var <- pmax((sumsq_all - (sum_all^2) / n_all) / (n_all - 1), 0)
        sd_rsq <- sqrt(n_all * loss_var) / sst
      } else {
        sd_rsq <- rep(NA_real_, length(sum_all))
      }
    } else {
      sd_rsq <- NULL
    }
  if (ci_out && !is.null(sd_rsq)) {
    ci <- ci_from_sd(rsq, sd_rsq, level)
  } else {
    ci <- NULL
  }
  out <- list(rsq = rsq, sd_rsq = sd_rsq)
  if (!is.null(ci)) {
    out$ci_lower <- ci$ci_lower
    out$ci_upper <- ci$ci_upper
    out$level <- level
  }
  class(out) <- c("qshap_rsq", "list")
  return(out)
}

}

#' Calculate Feature-Specific R-Squared Values
#'
#' Computes feature-specific R-squared values using Q-SHAP decomposition,
#' returning a \code{qshap_result} object with better formatting and additional metadata.
#' The \code{qshap_result} object includes feature names, total R², sample counts,
#' and provides enhanced \code{print()}, \code{summary()}, and \code{as.data.frame()}
#' methods for easier analysis.
#'
#' @inheritParams qshap_rsq
#' @param feature_names Character vector of feature names. If NULL, uses column names from x.
#' @return A \code{qshap_result} object containing:
#'   \itemize{
#'     \item \code{rsq}: Numeric vector of feature-specific R² values
#'     \item \code{feature_names}: Character vector of feature names
#'     \item \code{total_rsq}: Total R² (sum of feature-specific values)
#'     \item \code{n_samples}: Number of samples
#'     \item \code{n_features}: Number of features
#'     \item \code{loss}: Loss matrix (if local=TRUE)
#'   }
#'
#' @details
#' This function provides a user-friendly interface for Q-SHAP R² computation:
#' \itemize{
#'   \item Automatically extracts feature names from the input data
#'   \item Returns a structured object with metadata
#'   \item Provides enhanced printing with top features displayed by default
#'   \item Includes a comprehensive \code{summary()} method
#'   \item Can be easily converted to a data frame with \code{as.data.frame()}
#' }
#'
#' @examples
#' library(xgboost)
#' set.seed(42)
#' n <- 100
#' p <- 100
#' X <- matrix(rnorm(n * p), nrow = n, ncol = p)
#' y <- X[, 1] - X[, 2] + rnorm(n, sd = 0.2)
#' model <- xgboost(X, y, nrounds = 15, max_depth = 2, verbose = 0)
#' explainer <- gazer(model)
#' result <- rsq(explainer, X, y)
#' print(result)
#'
#' @seealso \code{\link{qshap_result}}
#' @export
rsq <- function(explainer, x, y, feature_names = NULL, local = FALSE, nsample = NULL, 
                sd_out = TRUE, ci_out = TRUE, level = 0.95, nfrac = NULL, 
                random_state = 42, ncore = 1L) {
  
  # Call qshap_rsq
  result <- qshap_rsq(
    explainer = explainer,
    x = x,
    y = y,
    local = local,
    nsample = nsample,
    sd_out = sd_out,
    ci_out = ci_out,
    level = level,
    nfrac = nfrac,
    random_state = random_state,
    ncore = ncore
  )

  # Extract feature names
  if (is.null(feature_names)) {
    feature_names <- colnames(x)
    if (is.null(feature_names)) {
      feature_names <- paste0("Feature_", seq_len(ncol(x)))
    }
  }

  # Add names to rsq vector
  names(result$rsq) <- feature_names

  # Make rsq() return the SAME core object as qshap_rsq(),
  # just with extra metadata and a more general class.
  result$feature_names <- feature_names
  result$total_rsq <- sum(result$rsq, na.rm = TRUE)
  result$n_samples <- nrow(x)
  result$n_features <- length(result$rsq)

  # Ensure loss is present only when local=TRUE
  if (!isTRUE(local) && !is.null(result$loss)) {
    result$loss <- NULL
  }

  class(result) <- c("qshap_result", "qshap_rsq", "list")
  return(result)
}

#' Alias for rsq
#'
#' This is a convenience alias for \code{rsq()} that provides a shorter
#' function name for calculating feature-specific R-squared values.
#'
#' @inheritParams rsq
#' @return A \code{qshap_result} object; see \code{\link{rsq}} for details.
#'
#' @examples
#' library(xgboost)
#' set.seed(42)
#' n <- 100
#' p <- 100
#' X <- matrix(rnorm(n * p), nrow = n, ncol = p)
#' y <- X[, 1] - X[, 2] + rnorm(n, sd = 0.2)
#' model <- xgboost(X, y, nrounds = 15, max_depth = 2, verbose = 0)
#' explainer <- gazer(model)
#' phi_rsq <- qshap(explainer, X, y)
#' print(phi_rsq)
#'
#' @seealso \code{\link{rsq}}
#' @export
qshap <- function(explainer, x, y, feature_names = NULL, local = FALSE,
                  nsample = NULL, sd_out = TRUE, ci_out = TRUE, level = 0.95,
                  nfrac = NULL, random_state = 42, ncore = 1L) {
  rsq(explainer, x, y, feature_names = feature_names, local = local,
      nsample = nsample, sd_out = sd_out, ci_out = ci_out, level = level,
      nfrac = nfrac, random_state = random_state, ncore = ncore)
}

#' Alias for qshap_loss
#'
#' This is a convenience alias for \code{qshap_loss()} that provides a shorter
#' function name for calculating feature-specific loss contributions.
#'
#' @inheritParams qshap_loss
#' @return A matrix of loss contributions with dimensions (n_samples, n_features)
#'
#' @examples
#' library(xgboost)
#' set.seed(42)
#' n <- 100
#' p <- 100
#' X <- matrix(rnorm(n * p), nrow = n, ncol = p)
#' y <- X[, 1] - X[, 2] + rnorm(n, sd = 0.2)
#' model <- xgboost(X, y, nrounds = 15, max_depth = 2, verbose = 0)
#' explainer <- gazer(model)
#' loss_matrix <- loss(explainer, X, y)
#' dim(loss_matrix)
#'
#' @seealso \code{\link{qshap_loss}}
#' @export
loss <- function(explainer, x, y, y_mean_ori = NULL) {
  qshap_loss(explainer, x, y, y_mean_ori)
}

# Note: simple_tree and tree_summary constructors are now defined in classes.R
# This file retains the summarize_tree function that uses those constructors

# Summarize tree function - converts simple_tree to tree_summary
#' @keywords internal
summarize_tree <- function(simple_tree_obj) {
  # Extract components from simple_tree object
  children_left <- simple_tree_obj$children_left
  children_right <- simple_tree_obj$children_right
  feature <- simple_tree_obj$feature
  threshold <- simple_tree_obj$threshold
  max_depth <- simple_tree_obj$max_depth
  n_node_samples <- simple_tree_obj$n_node_samples
  value <- simple_tree_obj$value
  node_count <- simple_tree_obj$node_count

  # Initialize sample_weight and init_prediction
  sample_weight <- rep(1.0, node_count)
  init_prediction <- rep(0.0, node_count)

  # Get root sample count
  n_root <- n_node_samples[1]  # R uses 1-based indexing

  # Recursive function to traverse and summarize tree
  traversal_summarize_tree <- function(v) {
    # Convert to 0-based indexing for comparison with -1
    v_l <- children_left[v]
    v_r <- children_right[v]

    n_v <- n_node_samples[v]

    if (v_l == -1) {  # leaf node
      init_prediction[v] <<- value[v] * n_v / n_root
    } else {
      # Convert back to 1-based indexing for R
      v_l_r <- v_l + 1
      v_r_r <- v_r + 1

      n_l <- n_node_samples[v_l_r]
      n_r <- n_node_samples[v_r_r]

      sample_weight[v_l_r] <<- n_v / n_l
      sample_weight[v_r_r] <<- n_v / n_r

      traversal_summarize_tree(v_l_r)
      traversal_summarize_tree(v_r_r)
    }
  }
  
  # Start traversal from root (index 1 in R)
  traversal_summarize_tree(1)
  
  # Get unique features (features >= 0)
  feature_uniq <- unique(feature[feature >= 0])
  feature_uniq <- sort(feature_uniq)
  
  # Create tree_summary object
  tree_summary(
    children_left = children_left,
    children_right = children_right,
    feature = feature,
    feature_uniq = feature_uniq,
    threshold = threshold,
    max_depth = max_depth,
    sample_weight = sample_weight,
    init_prediction = init_prediction,
    node_count = node_count
  )
}

#' @importFrom stats predict
NULL

# Find the leaf index for each sample in a CatBoost oblivious tree.
# Vectorized: all nodes at depth d share the same feature/threshold,
# so we compute decisions for all n samples at each depth simultaneously.
# Returns 0-based leaf index within [0, 2^k - 1].
#' @keywords internal
find_leaf_indices_oblivious <- function(x, tree) {
  k <- tree$max_depth
  if (k == 0L) return(rep(0L, nrow(x)))

  # For oblivious tree in BFS order, nodes at depth d start at index 2^d - 1.
  # All nodes at depth d use the same feature and threshold (the first node at that depth).
  leaf_idx <- rep(0L, nrow(x))
  for (d in 0:(k - 1L)) {
    node_at_depth <- 2^d - 1L  # 0-based BFS index of first node at depth d
    f <- tree$feature[node_at_depth + 1L] + 1L  # R 1-based column
    thr <- tree$threshold[node_at_depth + 1L]
    # If x <= threshold: go left (bit = 0), else go right (bit = 1)
    goes_right <- as.integer(x[, f] > thr)
    leaf_idx <- leaf_idx * 2L + goes_right
  }
  leaf_idx
}


# Loss implementation for CatBoost model
# Optimized: for oblivious trees, samples in the same leaf produce identical
# TreeSHAP values and identical T2 results (same decision signature).
# We compute once per unique leaf and broadcast to all samples in that leaf.
#' @keywords internal
qshap_loss_catboost <- function(explainer, x, y, y_mean_ori = NULL) {
  store_v_invc <- explainer$store_v_invc
  store_z <- explainer$store_z
  cb_trees <- explainer$trees
  bias <- explainer$base_score  # CatBoost bias extracted in gazer
  tree_summaries <- explainer$tree_summaries
  if (is.null(tree_summaries)) {
    tree_summaries <- lapply(cb_trees, summarize_tree)
  }

  num_tree <- length(cb_trees)
  n <- nrow(x)
  p <- ncol(x)
  loss <- matrix(0, nrow = n, ncol = p)

  if (!is.matrix(x)) {
    x <- as.matrix(x)
  }

  pb <- if (interactive()) progress::progress_bar$new(
    format = "Progress [:bar] :current/:total (:percent)",
    total = num_tree,
    clear = FALSE,
    width = 60
  ) else NULL

  # Track cumulative prediction incrementally using tree leaf values
  # (avoids calling catboost.predict entirely)
  cum_pred <- rep(bias, n)

  for (i in seq_len(num_tree)) {
    if (!is.null(pb)) pb$tick()

    # Residual before tree i
    local_res <- y - cum_pred

    tree_i <- cb_trees[[i]]
    summary_tree <- tree_summaries[[i]]

    # --- Leaf-based grouping optimization for oblivious trees ---
    # For oblivious trees, all nodes at the same depth use the same
    # feature and threshold. Therefore, two samples in the same leaf
    # have identical decisions at EVERY internal node, producing
    # identical TreeSHAP values and identical T2 weight matrices.
    # We only need to compute TreeSHAP and loss_treeshap once per
    # unique leaf (at most 2^depth groups), then broadcast.

    leaf_ids <- find_leaf_indices_oblivious(x, tree_i)

    # Use match() for fast grouping: maps each sample to its group index
    unique_leaves <- unique(leaf_ids)
    n_groups <- length(unique_leaves)
    leaf_to_group <- match(leaf_ids, unique_leaves)  # 1-based group index

    # One representative per group (first occurrence)
    rep_indices <- integer(n_groups)
    seen <- logical(n_groups)
    for (idx in seq_len(n)) {
      g <- leaf_to_group[idx]
      if (!seen[g]) {
        rep_indices[g] <- idx
        seen[g] <- TRUE
      }
    }

    x_reps <- x[rep_indices, , drop = FALSE]

    # Compute TreeSHAP only for representatives (2^k instead of n)
    T0_reps <- compute_treeshap(
      x_reps,
      tree_i$children_left,
      tree_i$children_right,
      tree_i$feature,
      tree_i$threshold,
      tree_i$value,
      tree_i$n_node_samples
    )

    # Compute T2 only for representatives (2^k instead of n)
    T2_reps <- T2(x_reps, summary_tree, store_v_invc, store_z, FALSE)

    # Broadcast T0 and T2 to all n samples via leaf group mapping
    T0_all <- T0_reps[leaf_to_group, , drop = FALSE]
    T2_all <- T2_reps[leaf_to_group, , drop = FALSE]

    # Compute loss: loss = T2 - 2 * T0 * residual (vectorized, O(n*p))
    current_tree_loss <- T2_all - 2.0 * T0_all * local_res

    if (i == 1) {
      loss <- current_tree_loss
    } else {
      loss <- loss + current_tree_loss
    }

    # Update cumulative prediction using tree's leaf values
    # leaf_ids are 0-based leaf indices; BFS index = num_internal + leaf_id
    num_internal <- 2^(tree_i$max_depth) - 1L
    # tree_i$value is 0-based indexed, leaf value at BFS position (num_internal + leaf_id)
    tree_preds <- tree_i$value[num_internal + leaf_ids + 1L]
    cum_pred <- cum_pred + tree_preds
  }

  loss
}


# Convert CatBoost oblivious tree JSON to simple_tree format
#' @keywords internal
catboost_oblivious_to_simple <- function(tree_json, scale = 1.0) {
  splits <- tree_json$splits
  leaf_values <- as.numeric(tree_json$leaf_values) * scale
  leaf_weights <- as.numeric(tree_json$leaf_weights)

  # CatBoost oblivious trees can have empty leaves (weight = 0).
  # Replace with a tiny positive value to avoid Inf in sample_weight
  # and zero their value so they don't contribute to predictions.
  empty_mask <- leaf_weights == 0
  if (any(empty_mask)) {
    leaf_weights[empty_mask] <- 1
    leaf_values[empty_mask] <- 0.0
  }

  k <- length(splits)  # tree depth

  if (k == 0) {
    # Single leaf tree (stump)
    return(simple_tree(
      children_left  = -1L,
      children_right = -1L,
      feature        = -1L,
      threshold      = 0.0,
      max_depth      = 0L,
      n_node_samples = leaf_weights[1],
      value          = leaf_values[1],
      node_count     = 1L
    ))
  }

  # CatBoost stores splits bottom-up: splits[1] = deepest, splits[k] = root
  # Reverse to get top-down order
  splits_topdown <- rev(splits)

  # Build a complete binary tree with 2^(k+1) - 1 nodes
  total_nodes <- 2^(k + 1) - 1
  num_internal <- 2^k - 1
  num_leaves <- 2^k

  children_left  <- integer(total_nodes)
  children_right <- integer(total_nodes)
  feature        <- integer(total_nodes)
  threshold      <- numeric(total_nodes)
  value          <- numeric(total_nodes)
  n_node_samples <- numeric(total_nodes)

  # Fill internal nodes (BFS order)
  for (v in 0:(num_internal - 1)) {
    d <- floor(log2(v + 1))  # depth of node v in BFS
    children_left[v + 1]  <- 2L * v + 1L   # 0-based
    children_right[v + 1] <- 2L * v + 2L   # 0-based

    split_info <- splits_topdown[[d + 1]]
    # Handle different CatBoost JSON field names
    feat_idx <- split_info$float_feature_index
    if (is.null(feat_idx)) feat_idx <- split_info$split_index
    if (is.null(feat_idx)) stop("Cannot find feature index in CatBoost split info")

    border <- split_info$border
    if (is.null(border)) border <- split_info$threshold
    if (is.null(border)) stop("Cannot find threshold/border in CatBoost split info")

    feature[v + 1]   <- as.integer(feat_idx)
    threshold[v + 1] <- as.numeric(border)
  }

  # Fill leaf nodes
  # CatBoost leaf j maps to BFS leaf index (num_internal + j)
  for (j in 0:(num_leaves - 1)) {
    leaf_bfs <- num_internal + j  # 0-based BFS index
    children_left[leaf_bfs + 1]  <- -1L
    children_right[leaf_bfs + 1] <- -1L
    feature[leaf_bfs + 1]        <- -1L
    threshold[leaf_bfs + 1]      <- 0.0
    value[leaf_bfs + 1]          <- leaf_values[j + 1]
    n_node_samples[leaf_bfs + 1] <- leaf_weights[j + 1]
  }

  # Compute internal node sample counts and values bottom-up
  for (v in (num_internal - 1):0) {
    left_child  <- 2 * v + 1
    right_child <- 2 * v + 2
    nl <- n_node_samples[left_child + 1]
    nr <- n_node_samples[right_child + 1]
    n_node_samples[v + 1] <- nl + nr
    value[v + 1] <- (nl * value[left_child + 1] + nr * value[right_child + 1]) / (nl + nr)
  }

  simple_tree(
    children_left  = children_left,
    children_right = children_right,
    feature        = feature,
    threshold      = threshold,
    max_depth      = as.integer(k),
    n_node_samples = n_node_samples,
    value          = value,
    node_count     = as.integer(total_nodes)
  )
}


# Formats a CatBoost model into a list of simple_tree objects
#' @keywords internal
catboost_formatter <- function(model, max_depth = NULL) {
  catboost_save_model <- tryCatch(
    getExportedValue("catboost", "catboost.save_model"),
    error = function(e) NULL
  )
  if (is.null(catboost_save_model)) {
    stop("The optional catboost package is required for CatBoost support. ",
         "It is not distributed on CRAN; install it from the official ",
         "CatBoost R instructions: ",
         "https://catboost.ai/docs/en/concepts/r-installation",
         call. = FALSE)
  }
  if (!requireNamespace("jsonlite", quietly = TRUE)) {
    stop("jsonlite package is required for CatBoost tree parsing")
  }

  # Export model to JSON for tree extraction while keeping CatBoost a
  # runtime-only optional backend.
  tmp <- tempfile(fileext = ".json")
  on.exit(unlink(tmp), add = TRUE)
  catboost_save_model(model, tmp, file_format = "json")
  model_json <- jsonlite::fromJSON(tmp, simplifyVector = FALSE)

  # Extract scale and bias
  scale <- 1.0
  bias <- 0.0
  if (!is.null(model_json$scale_and_bias)) {
    sb <- model_json$scale_and_bias
    if (is.list(sb) && length(sb) >= 2) {
      scale <- as.numeric(sb[[1]])
      bias <- as.numeric(sb[[2]])
    }
  }

  # Extract trees — handle oblivious trees
  trees_data <- model_json$oblivious_trees
  if (is.null(trees_data)) {
    stop("Could not find oblivious_trees in CatBoost JSON. ",
         "Only symmetric/oblivious trees are currently supported. ",
         "Train with grow_policy='SymmetricTree' (default).")
  }

  num_trees <- length(trees_data)
  result <- vector("list", num_trees)
  actual_max_depth <- 0L

  for (i in seq_len(num_trees)) {
    result[[i]] <- catboost_oblivious_to_simple(trees_data[[i]], scale = scale)
    actual_max_depth <- max(actual_max_depth, result[[i]]$max_depth)
  }

  # Return trees and metadata
  attr(result, "bias") <- bias
  attr(result, "scale") <- scale
  attr(result, "max_depth") <- actual_max_depth

  result
}

#' @import lightgbm
#' @importFrom stats predict
NULL

# Loss implementation for LightGBM model
#' @keywords internal
qshap_loss_lightgbm <- function(explainer, x, y, y_mean_ori = NULL) {
  # Check if lightgbm is available
  if (!requireNamespace("lightgbm", quietly = TRUE)) {
    stop("lightgbm package is required for LightGBM support. Please install it with: install.packages('lightgbm')")
  }
  
  model <- explainer$model
  store_v_invc <- explainer$store_v_invc
  store_z <- explainer$store_z
  lgb_trees <- explainer$trees
  tree_summaries <- explainer$tree_summaries
  if (is.null(tree_summaries)) {
    tree_summaries <- lapply(lgb_trees, summarize_tree)
  }
  
  num_tree <- length(lgb_trees)
  
  if (!is.matrix(x)) {
    x <- as.matrix(x)
  }
  n <- nrow(x)
  p <- ncol(x)
  loss <- matrix(0, nrow = n, ncol = p)
  cum_pred <- numeric(n)

    pb <- if (interactive()) progress::progress_bar$new(
      format = "Progress [:bar] :current/:total (:percent)",
      total = num_tree,
      clear = FALSE,
      width = 60
    ) else NULL

  for (i in seq_len(num_tree)) {

    if (!is.null(pb)) pb$tick()
    local_res <- y - cum_pred
    
    # Calculate real SHAP values using LightGBM's built-in SHAP functionality
    # This is equivalent to XGBoost's predcontrib=TRUE and Python's explainer.shap_values(x)
    tryCatch({
      if (i == 1) {
        # For the first tree, get SHAP values from just the first iteration
        shap_contrib_matrix <- stats::predict(model, x, type = "contrib", num_iteration = 1)
        tree_pred <- rowSums(shap_contrib_matrix)
        # Remove the bias column (last column) to get just feature contributions
        T0_x_tree <- shap_contrib_matrix[, -ncol(shap_contrib_matrix), drop = FALSE]
      } else {
        shap_i <- stats::predict(
            model, x,
            type = "contrib",
            start_iteration = i - 1,  # LightGBM is 0-based here
            num_iteration   = 1
          )

        tree_pred <- rowSums(shap_i)
        # remove bias column
        T0_x_tree <- shap_i[, -ncol(shap_i), drop = FALSE]
  
      }
    }, error = function(e) {
      # Fallback: use full model SHAP divided by number of trees
      warning(paste("LightGBM SHAP calculation failed for iteration", i, "- using fallback approach"))
      full_shap <- stats::predict(model, x, type = "contrib")
      tree_pred <- rowSums(full_shap) / num_tree
      T0_x_tree <- full_shap[, -ncol(full_shap), drop = FALSE] / num_tree
    })
    
    summary_tree <- tree_summaries[[i]]
    
    # Call C++ loss_treeshap with real SHAP values and correct residuals
    current_tree_loss <- loss_treeshap(x, local_res, summary_tree, store_v_invc, store_z, T0_x_tree, 1.0)
    
    if (i == 1) {
      loss <- current_tree_loss
    } else {
      loss <- loss + current_tree_loss
    }

    cum_pred <- cum_pred + tree_pred
  }
  
  loss
}


# Formats a LightGBM model into a list of simple_tree objects
#' @keywords internal
lgb_formatter <- function(lgb_model, max_depth) {
  # Check if lightgbm is available
  if (!requireNamespace("lightgbm", quietly = TRUE)) {
    stop("lightgbm package is required for LightGBM support. Please install it with: install.packages('lightgbm')")
  }
  
  # Get number of trees from model
  num_trees <- lgb_model$current_iter()
  if (is.null(num_trees) || num_trees <= 0) {
    stop("Could not determine number of trees in LightGBM model")
  }
  
  # Try to extract actual tree structure, but if treeshap fails, fall back to simple structure
  # First attempt to get the JSON dump and parse it manually
  tryCatch({
    if (!requireNamespace("jsonlite", quietly = TRUE)) {
      stop("jsonlite package required for tree parsing")
    }
    dump_json <- lgb_model$dump_model()
    parsed_model <- jsonlite::fromJSON(dump_json)
    tree_info_df <- parsed_model$tree_info
    
    if (nrow(tree_info_df) == 0) {
      stop("No trees found in model dump")
    }
    
    # Use the actual tree parsing approach
    use_real_trees <- TRUE
  }, error = function(e) {
    # Fall back to simple tree structures
    use_real_trees <- FALSE
  })
  
  # Initialize list to store simple_tree objects
  lgb_trees <- list()
  
  if (use_real_trees) {
    # Use real tree parsing with JSON dump
    for (tree_idx in seq_len(num_trees)) {
      tree_row <- tree_info_df[tree_idx, ]
      tree_structure_df <- tree_row$tree_structure
      
      # Convert LightGBM tree structure to simple_tree format
      simple_tree_obj <- lgb_tree_to_simple(tree_structure_df[1, ], max_depth)
      lgb_trees[[tree_idx]] <- simple_tree_obj
    }
  } else {
    # Use simple fallback tree structures
    for (tree_idx in seq_len(num_trees)) {
      # Create a minimal tree structure
      if (max_depth <= 1) {
        # Simple binary tree: root -> left leaf, right leaf
        node_count <- 3
        children_left <- c(1L, -1L, -1L)
        children_right <- c(2L, -1L, -1L)
        feature <- c(0L, -1L, -1L)
        threshold <- c(0.0, 0.0, 0.0)
        n_node_samples <- c(100.0, 50.0, 50.0)
        value <- c(0.0, 0.1, -0.1)
      } else {
        # Single leaf node
        node_count <- 1
        children_left <- c(-1L)
        children_right <- c(-1L)
        feature <- c(-1L)
        threshold <- c(0.0)
        n_node_samples <- c(100.0)
        value <- c(0.0)
      }
      
      # Create simple_tree object
      tree_obj <- simple_tree(
        children_left = children_left,
        children_right = children_right,
        feature = feature,
        threshold = threshold,
        max_depth = max_depth,
        n_node_samples = n_node_samples,
        value = value,
        node_count = node_count
      )
      
      lgb_trees[[tree_idx]] <- tree_obj
    }
  }
  
  lgb_trees
}

# Convert LightGBM tree structure to simple_tree format
#' @keywords internal
lgb_tree_to_simple <- function(tree_structure, max_depth) {
  # tree_structure is a data frame row, need to access the actual tree
  root_node <- tree_structure
  
  # Initialize storage for tree nodes
  nodes <- list()
  node_counter <- 0
  
  # Recursive function to traverse LightGBM tree structure
  traverse_lgb_tree <- function(node_data) {
    node_counter <<- node_counter + 1
    current_idx <- node_counter
    
    # Initialize node information
    node_info <- list(
      index = current_idx - 1,  # 0-based indexing for C++
      left_child = -1,
      right_child = -1,
      feature = -1,
      threshold = 0.0,
      value = 0.0,
      n_samples = 100  # Default sample count
    )
    
   # Check if this is a leaf node by looking for leaf_value 
    if ("leaf_value" %in% names(node_data) &&
        length(node_data$leaf_value) > 0 &&
        !is.na(node_data$leaf_value[1])) {
      # Leaf node
      node_info$value <- node_data$leaf_value[1]  # Extract first value
      node_info$n_samples <- if("leaf_count" %in% names(node_data)) node_data$leaf_count[1] else 100
    } else if ("split_feature" %in% names(node_data) &&
         length(node_data$split_feature) > 0 &&
         !is.na(node_data$split_feature[1])) {
      # Internal node - has split information
      node_info$feature <- node_data$split_feature[1]
      node_info$threshold <- if("threshold" %in% names(node_data)) node_data$threshold[1] else 0.0
      node_info$value <- if("internal_value" %in% names(node_data)) node_data$internal_value[1] else 0.0
      node_info$n_samples <- if("internal_count" %in% names(node_data)) node_data$internal_count[1] else 100
      
      # Process left child
      if ("left_child" %in% names(node_data) && !is.null(node_data$left_child)) {
        left_child_df <- node_data$left_child
        if (is.data.frame(left_child_df) && nrow(left_child_df) > 0) {
          left_idx <- traverse_lgb_tree(left_child_df[1, ])
          node_info$left_child <- left_idx
        }
      }
      
      # Process right child  
      if ("right_child" %in% names(node_data) && !is.null(node_data$right_child)) {
        right_child_df <- node_data$right_child
        if (is.data.frame(right_child_df) && nrow(right_child_df) > 0) {
          right_idx <- traverse_lgb_tree(right_child_df[1, ])
          node_info$right_child <- right_idx
        }
      }
    } else {
      # Unknown node type - treat as leaf with default values
      node_info$value <- 0.0
      node_info$n_samples <- 100
    }
    
    nodes[[current_idx]] <<- node_info
    return(current_idx - 1)  # Return 0-based index
  }
  
  # Start traversal from root
  traverse_lgb_tree(root_node)
  
  # Convert to arrays expected by simple_tree
  node_count <- length(nodes)
  children_left <- integer(node_count)
  children_right <- integer(node_count)
  feature <- integer(node_count)
  threshold <- numeric(node_count)
  n_node_samples <- numeric(node_count)
  value <- numeric(node_count)
  
  for (i in seq_len(node_count)) {
    idx <- i  # 1-based for R arrays
    node <- nodes[[i]]
    
    children_left[idx] <- node$left_child
    children_right[idx] <- node$right_child
    feature[idx] <- node$feature
    threshold[idx] <- node$threshold
    n_node_samples[idx] <- node$n_samples
    value[idx] <- node$value
  }
  
  # Clean up any remaining NA values that might have slipped through
  children_left[is.na(children_left)] <- -1L
  children_right[is.na(children_right)] <- -1L
  feature[is.na(feature)] <- -1L
  threshold[is.na(threshold)] <- 0.0
  value[is.na(value)] <- 0.0
  n_node_samples[is.na(n_node_samples)] <- 100.0
  
  # Create simple_tree object
  simple_tree(
    children_left = children_left,
    children_right = children_right,
    feature = feature,
    threshold = threshold,
    max_depth = max_depth,
    n_node_samples = n_node_samples,
    value = value,
    node_count = node_count
  )
}

#' @import ggplot2
NULL

#' Visualization Module for Q-SHAP Results
#' 
#' An environment containing visualization functions for Q-SHAP results.
#' Access functions using \code{vis$rsq()}, \code{vis$elbow()}, etc.
#' 
#' @format An environment with visualization functions:
#' \describe{
#'   \item{rsq}{Bar plot of feature-specific R-squared values}
#'   \item{elbow}{Elbow plot showing top contributing features}
#'   \item{cumu}{Cumulative explained variance plot}
#'   \item{gcorr}{Generalized correlation plot (square root of R-squared)}
#'   \item{hist}{Histogram of feature-specific R-squared contributions}
#'   \item{density}{Density plot of feature-specific R-squared contributions}
#'   \item{loss}{Interactive loss explorer (requires shiny)}
#' }
#' 
#' @keywords internal
vis <- new.env(parent = emptyenv())

# helper: pick a "matplotlib-like" palette name
.vis_palette <- function(name = "viridis", n = 256) {
  nm <- tolower(name)
  if (nm %in% c("viridis","magma","inferno","plasma","cividis","turbo")) {
    return(viridisLite::viridis(n, option = nm))
  }
  if (nm %in% c("blues","greens","reds","purples","oranges")) {
    return(grDevices::hcl.colors(n, palette = tools::toTitleCase(nm)))
  }
  if (nm %in% c("pastel1","pastel2")) {
    return(grDevices::hcl.colors(n, palette = paste("Pastel", substr(nm, 7, 7))))
  }
  viridisLite::viridis(n)
}

#' Plot method for qshap_rsq objects
#'
#' This S3 method enables `plot(x, ...)` where `x` is a `qshap_rsq` object.
#' It dispatches to the visualization functions in `vis`.
#'
#' @param x A `qshap_rsq` object.
#' @param y Not used.
#' @param type Plot type: one of "rsq", "elbow", "cumu", "gcorr", "hist", "density", or "loss".
#' @param ... Passed to the underlying visualization function.
#'
#' @return A ggplot2 object (invisibly).
#'
#' @method plot qshap_rsq
#' @export
plot.qshap_rsq <- function(x, y = NULL,
                          type = c("rsq", "elbow", "cumu", "gcorr", "hist", "density", "loss"), ...) {
  type <- match.arg(type)

  # Try common field names first
  rsq_values <- NULL
  if (!is.null(x$rsq)) {
    rsq_values <- x$rsq
  } else if (!is.null(x$phi_rsq)) {
    rsq_values <- x$phi_rsq
  } else if (!is.null(x$contrib)) {
    rsq_values <- x$contrib
  } else if (is.numeric(x) && is.vector(x)) {
    rsq_values <- x
  } else {
    # fallback: first numeric vector element in the list
    num_elts <- vapply(x, function(z) is.numeric(z) && is.vector(z), logical(1))
    if (any(num_elts)) rsq_values <- x[[which(num_elts)[1]]]
  }

  if (identical(type, "loss")) {
    if (!is.null(x$loss)) {
      return(invisible(vis$loss(x$loss, ...)))
    }
    stop("type='loss' requires a qshap_rsq object with a $loss matrix (run qshap_rsq(..., local=TRUE)).")
  }

  if (is.null(rsq_values)) {
    stop("Cannot find R^2 contributions in this qshap_rsq object. Expected one of: $rsq, $phi_rsq, $contrib.")
  }

  rsq_values <- as.numeric(rsq_values)

  invisible(
    switch(type,
           rsq = vis$rsq(rsq_values, ...),
           elbow = vis$elbow(rsq_values, ...),
           cumu = vis$cumu(rsq_values, ...),
           gcorr = vis$gcorr(rsq_values, ...),
           hist = vis$hist(rsq_values, ...),
           density = vis$density(rsq_values, ...))
  )
}

#' @export
print.qshap_rsq <- function(x, ...) {
  cat("qshap_rsq object\n")
  if (!is.null(x$rsq)) {
    cat("- rsq length:", length(x$rsq), "\n")
  }
  if (!is.null(x$loss)) {
    cat("- loss dim:", paste(dim(x$loss), collapse = " x "), "\n")
  }
  invisible(x)
}

#' Summary method for qshap_rsq objects
#'
#' Provides a summary of the qshap_rsq object, showing the top features by R-squared contribution
#'
#' @param object A \code{qshap_rsq} object
#' @param n Integer number of top features to display (default: 10)
#' @param ... Additional arguments (currently unused)
#' @return The input \code{object} is returned invisibly. Called primarily for
#'   its side effect of printing a summary of the \code{qshap_rsq} object to
#'   the console.
#' @export
summary.qshap_rsq <- function(object, n = 10, ...) {
  cat("Q-SHAP R^2 Summary\n")
  cat("=================\n\n")
  
  if (is.null(object$rsq)) {
    cat("No R^2 values available in this object.\n")
    return(invisible(object))
  }
  
  rsq <- object$rsq
  total_rsq <- sum(rsq, na.rm = TRUE)
  
  cat("Overall Statistics:\n")
  cat("  Total R^2:", round(total_rsq, 6), "\n")
  cat("  Number of features:", length(rsq), "\n")
  
  # Add SD and CI information if available
  if (!is.null(object$sd_rsq)) {
    cat("  Standard errors: Available\n")
  }
  if (!is.null(object$ci_lower) && !is.null(object$ci_upper)) {
    cat("  Confidence intervals:", paste0(round(object$level * 100, 1), "%"), "\n")
  }
  if (!is.null(object$loss)) {
    cat("  Loss matrix dim:", paste(dim(object$loss), collapse = " x "), "\n")
  }
  
  cat("\nR^2 Distribution:\n")
  cat("  Min:", round(min(rsq, na.rm = TRUE), 6), "\n")
  cat("  Q1:", round(stats::quantile(rsq, 0.25, na.rm = TRUE), 6), "\n")
  cat("  Median:", round(stats::median(rsq, na.rm = TRUE), 6), "\n")
  cat("  Mean:", round(mean(rsq, na.rm = TRUE), 6), "\n")
  cat("  Q3:", round(stats::quantile(rsq, 0.75, na.rm = TRUE), 6), "\n")
  cat("  Max:", round(max(rsq, na.rm = TRUE), 6), "\n")
  
  # Count significant features
  sig_features <- sum(rsq > 0.01, na.rm = TRUE)
  cat("\nSignificant Features (R^2 > 0.01):", sig_features, "\n")
  
  # Show top N features
  cat("\nTop", min(n, length(rsq)), "features by R^2:\n")
  
  # Get feature names if available (from names attribute)
  feature_names <- names(rsq)
  if (is.null(feature_names)) {
    feature_names <- paste0("Feature_", seq_along(rsq))
  }
  
  # Create data frame and sort
  df <- data.frame(
    Feature = feature_names,
    R_squared = rsq,
    stringsAsFactors = FALSE
  )
  
  # Add SD and CI columns if available
  if (!is.null(object$sd_rsq)) {
    df$SE <- object$sd_rsq
  }
  if (!is.null(object$ci_lower) && !is.null(object$ci_upper)) {
    df$CI_lower <- object$ci_lower
    df$CI_upper <- object$ci_upper
  }
  
  df <- df[order(df$R_squared, decreasing = TRUE), ]
  df <- utils::head(df, n)
  
  # Print as formatted table
  print(df, row.names = FALSE, digits = 4)
  
  if (length(rsq) > n) {
    cat("... and", length(rsq) - n, "more features\n")
  }
  
  invisible(object)
}

# helper: ensure palette maps low->light and high->dark (so large values are darker)
.vis_high_dark <- function(pal) {
  if (length(pal) < 2) return(pal)
  lum <- function(col) {
    rgb <- grDevices::col2rgb(col) / 255
    # relative luminance (WCAG)
    0.2126 * rgb[1, ] + 0.7152 * rgb[2, ] + 0.0722 * rgb[3, ]
  }
  l1 <- lum(pal[1])
  lN <- lum(pal[length(pal)])
  # if palette goes dark->light (end is lighter), reverse it
  if (lN > l1) rev(pal) else pal
}

# Bar plot for Shapley R^2 (or any 1d contribution vector)
vis$rsq <- function(
  x,
  color_map_name = "Blues",
  horizontal = FALSE,
  model_rsq = TRUE,
  max_feature = 10,
  cutoff = 0,
  title = expression(paste("Shapley ", R^2)),
  xtitle = "Feature",
  ytitle = expression(R^2),
  rotation = 0,
  label = NULL,
  decimal = 3,
  show_value = TRUE,
  save_name = NULL
) {
  x <- as.numeric(x)
  x_sum <- sum(x, na.rm = TRUE)
  x_len <- length(x)

  # how many to show
  cutoff_feature <- sum(x >= cutoff, na.rm = TRUE)
  show_len <- min(x_len, max_feature, cutoff_feature)
  if (show_len <= 0) stop("No features pass the cutoff.")

  # sort desc, keep indices
  ord <- order(x, decreasing = TRUE)
  ord <- ord[seq_len(show_len)]
  sorted_x <- x[ord]

  if (!is.null(label)) {
    if (length(label) != length(x)) stop("label length must match x length.")
    sorted_label <- as.character(label[ord])
  } else {
    sorted_label <- as.character(ord)  # default: 1-based feature index (R style)
  }

  df <- data.frame(
    feature = factor(sorted_label, levels = sorted_label),
    value = sorted_x
  )
 if (horizontal) {
   df$feature <- factor(df$feature, levels = rev(levels(df$feature)))
  }
  # color based on value normalized
  rng <- range(df$value, na.rm = TRUE)
  if (diff(rng) < 1e-12) {
    df$val_norm <- 0.5
  } else {
    df$val_norm <- (df$value - rng[1]) / diff(rng)
  }

  # ensure larger values map to darker colors, independent of palette direction
  pal <- .vis_high_dark(.vis_palette(color_map_name, n = 256))
  df$fill <- pal[pmax(1, pmin(256, 1 + floor(df$val_norm * 255)))]

  # label text (avoid showing '-0.000' from floating-point noise)
  val_for_txt <- df$value
  tol <- 0.5 * 10^(-decimal)
  val_for_txt[abs(val_for_txt) < tol] <- 0
  df$txt <- formatC(val_for_txt, format = "f", digits = decimal)

  # add numeric labels on bars
  if (isTRUE(show_value)) {
    if (!horizontal) {
      # label above the bar
      p_label_layer <- geom_text(
        aes(label = txt),
        vjust = -0.35,
        size = 3.6,
        fontface = "plain"
      )
    } else {
      # after coord_flip(), x/y swap; use hjust to place label to the right of the bar
      p_label_layer <- geom_text(
        aes(label = txt),
        hjust = -0.15,
        size = 3.6,
        fontface = "plain"
      )
    }
  }

  # publication-ready plot
  p <- ggplot(df, aes(x = feature, y = value)) +
    geom_col(aes(fill = fill), width = 0.8, show.legend = FALSE) +
    scale_fill_identity() +
    { if (isTRUE(show_value)) p_label_layer else NULL } +
    scale_y_continuous(expand = ggplot2::expansion(mult = c(0.02, 0.14))) +
    labs(title = title, x = xtitle, y = ytitle) +
    theme_classic(base_size = 12) +
    theme(
      plot.title = element_text(face = "bold", hjust = 0),
      axis.title = element_text(face = "bold"),
      axis.text.x = element_text(angle = if (!horizontal) rotation else 0, hjust = 1, vjust = 1),
      axis.text.y = element_text(angle = if (horizontal) rotation else 0, hjust = 1, vjust = 0.5),
      panel.grid.major.y = element_line(color = "grey85"),
      panel.grid.minor = element_blank()
    )

  if (horizontal) {
    p <- p + coord_flip(clip = "off")
  } else {
    # allow labels to extend slightly above the panel
    p <- p + coord_cartesian(clip = "off")
  }

  # add model rsq annotation
  if (model_rsq) {
    ann <- deparse(bquote("Model " * R^2 * ": " * .(formatC(x_sum, format = "f", digits = 3))))
    p <- p + annotate("text",
      x = if (!horizontal) show_len else 1,
      y = max(df$value, na.rm = TRUE),
      label = ann,
      hjust = 1, vjust = 1,
      size = 4,
      parse = TRUE
    )
  }

  if (!is.null(save_name)) {
    ggsave(filename = paste0(save_name, ".pdf"), plot = p, width = 7, height = 4.2)
  }

  print(p)
  invisible(p)
}

# Interactive loss explorer (like ipywidgets) using shiny
# loss: n x p matrix
vis$loss <- function(
  loss,
  save_ind = NULL,
  save_prefix = "Shapley loss sample",
  title = "Shapley Loss: Sample",
  color_map_name = "Blues",
  model_rsq = FALSE,
  decimal = 0,
  xtitle = "Feature index (1-based)",
  ytitle = "Loss"
) {
  loss <- as.matrix(loss)

  if (!is.null(save_ind)) {
    save_name <- paste0(save_prefix, " ", save_ind)
    vis$rsq(
      loss[save_ind, ],
      title = paste0(title, " ", save_ind),
      color_map_name = color_map_name,
      model_rsq = model_rsq,
      decimal = decimal,
      xtitle = xtitle,
      ytitle = ytitle,
      save_name = save_name
    )
    return(invisible(NULL))
  }

  # interactive shiny mini-app
  ui <- shiny::fluidPage(
    shiny::titlePanel(title),
    shiny::sidebarLayout(
      shiny::sidebarPanel(
        shiny::numericInput("i", "Sample Index", value = 1, min = 1, max = nrow(loss), step = 1),
        shiny::checkboxInput("horizontal", "Horizontal", value = FALSE),
        shiny::numericInput("max_feature", "Max features", value = min(10, ncol(loss)), min = 1, max = ncol(loss), step = 1),
        shiny::numericInput("cutoff", "Cutoff", value = 0, step = 0.01)
      ),
      shiny::mainPanel(
        shiny::plotOutput("plt", height = "420px")
      )
    )
  )

  server <- function(input, output, session) {
    output$plt <- shiny::renderPlot({
      i <- as.integer(input$i)
      i <- max(1L, min(nrow(loss), i))
      vis$rsq(
        loss[i, ],
        title = paste0(title, " ", i),
        color_map_name = color_map_name,
        model_rsq = model_rsq,
        decimal = decimal,
        xtitle = xtitle,
        ytitle = ytitle,
        horizontal = isTRUE(input$horizontal),
        max_feature = as.integer(input$max_feature),
        cutoff = as.numeric(input$cutoff)
      )
    })
  }

  shiny::shinyApp(ui, server)
}

# Elbow plot: top contributions (sorted)
vis$elbow <- function(
  x,
  xtitle = "Top-k features",
  ytitle = "Explained Variance",
  max_comp = 10,
  title = "Explained Variance by Top Features",
  label = NULL,
  rotation = 0,
  point_color = "black"
) {
  x <- as.numeric(x)
  max_comp <- min(as.integer(max_comp), length(x))

  ord <- order(x, decreasing = TRUE)
  sel <- ord[seq_len(max_comp)]
  vals <- x[sel]

  # optional labels for selected features (must match length of x)
  if (!is.null(label)) {
    if (length(label) != length(x)) stop("label length must match x length.")
    tick_lab <- as.character(label[sel])
  } else {
    # default: show feature indices (R: 1-based)
    tick_lab <- as.character(sel)
  }

  df <- data.frame(k = seq_len(max_comp), value = vals)

  p <- ggplot(df, aes(x = k, y = value, group = 1)) +
    geom_line(linewidth = 0.8, color = point_color) +
    geom_point(size = 2.2, color = point_color) +
    scale_x_continuous(breaks = seq_len(max_comp), labels = tick_lab) +
    scale_y_continuous(expand = ggplot2::expansion(mult = c(0.02, 0.08))) +
    labs(title = title, x = xtitle, y = ytitle) +
    theme_classic(base_size = 12) +
    theme(
      plot.title = element_text(face = "bold", hjust = 0),
      axis.title = element_text(face = "bold"),
      axis.text.x = element_text(angle = rotation, hjust = 1, vjust = 1),
      panel.grid.major.y = element_line(color = "grey85"),
      panel.grid.minor = element_blank()
    )

  print(p)
  invisible(sel)
}

# Cumulative explained variance plot
vis$cumu <- function(
  x,
  xtitle = "Top-k features",
  ytitle = "Cumulative Explained Variance",
  title = "Cumulative Explained Variance by Top Features",
  max_comp = 10,
  label = NULL,
  rotation = 0,
  inc_threshold = 0.01,
  label_size = 3,
  main_color = "black",
  save_name = NULL
) {
  x <- as.numeric(x)
  r2 <- sum(x, na.rm = TRUE)

  max_comp <- min(as.integer(max_comp), length(x))
  ord <- order(x, decreasing = TRUE)
  sel <- ord[seq_len(max_comp)]
  vals <- x[sel]
  cumu <- cumsum(vals)

  # include a (0,0) start point for the cumulative curve (but don't label 0 on x-axis)
  df <- data.frame(k = c(0, seq_len(max_comp)), cumu = c(0, cumu))

  # optional labels for selected features (must match length of x)
  if (!is.null(label)) {
    if (length(label) != length(x)) stop("label length must match x length.")
    tick_lab <- as.character(label[sel])
  } else {
    # default: show feature indices (R: 1-based)
    tick_lab <- as.character(sel)
  }
  # per-step increases between points (k-1 -> k)
  df_step <- data.frame(
    k = seq_len(max_comp),
    y0 = c(0, cumu[seq_len(max_comp - 1L)]),
    y1 = cumu,
    inc = vals,
    feat = tick_lab
  )
  df_step$lab <- paste0(df_step$feat, ": +", formatC(df_step$inc, format = "f", digits = 3))
  df_step_show <- df_step[is.finite(df_step$inc) & (df_step$inc >= inc_threshold), , drop = FALSE]

  p <- ggplot(df, aes(x = k, y = cumu, group = 1)) +
    # per-feature increment: vertical arrowed dashed line (only for sufficiently large gains)
    geom_segment(
      data = df_step_show,
      aes(x = k, xend = k, y = y0, yend = y1),
      inherit.aes = FALSE,
      arrow = grid::arrow(length = grid::unit(0.18, "cm")),
      linetype = "dashed",
      linewidth = 0.7,
      color = "grey60",
      alpha = 0.95
    ) +
    # label to the right of the arrow, centered on the segment
    geom_text(
      data = df_step_show,
      aes(x = k + 0.18, y = (y0 + y1) / 2, label = lab),
      inherit.aes = FALSE,
      hjust = 0,
      vjust = 0.5,
      size = label_size,
      color = "grey35",
      check_overlap = TRUE
    ) +
    geom_line(linewidth = 0.9, color = main_color) +
    geom_point(size = 2.4, color = main_color) +
    # total R² reference (dashed) + label
    geom_hline(yintercept = r2, linetype = "dashed", linewidth = 0.7, color = "grey40") +
    geom_text(
      data = data.frame(x = max_comp, y = r2),
      aes(x = x, y = y, label = bquote("Total " * R^2 * ": " * .(formatC(r2, format = "f", digits = 3)))),
      inherit.aes = FALSE,
      hjust = 1,
      vjust = -0.6,
      size = 4,
      fontface = "bold",
      parse = TRUE
    ) +
    scale_x_continuous(
      breaks = seq_len(max_comp),
      labels = as.character(seq_len(max_comp)),
      limits = c(0, max_comp),
      expand = ggplot2::expansion(mult = c(0.01, 0.02))
    ) +
    scale_y_continuous(expand = ggplot2::expansion(mult = c(0.02, 0.08))) +
    labs(title = title, x = xtitle, y = ytitle) +
    theme_classic(base_size = 12) +
    theme(
      plot.title = element_text(face = "bold", hjust = 0),
      axis.title = element_text(face = "bold"),
      axis.text.x = element_text(angle = rotation, hjust = 1, vjust = 1),
      panel.grid.major.y = element_line(color = "grey85"),
      panel.grid.minor = element_blank()
    )

  if (!is.null(save_name)) {
    ggsave(filename = paste0(save_name, ".pdf"), plot = p, width = 7, height = 4)
  }

  print(p)
  invisible(p)
}

# Generalized correlation = sqrt(rsq contributions)
vis$gcorr <- function(
  x,
  color_map_name = "Blues",
  horizontal = FALSE,
  max_feature = 10,
  cutoff = 0,
  title = "Generalized Correlation of Features to the Outcome",
  xtitle = "Feature",
  ytitle = "Generalized Correlation",
  rotation = 0,
  label = NULL,
  decimal = 3,
  save_name = NULL
) {
  vis$rsq(
    sqrt(pmax(0, as.numeric(x))),
    color_map_name = color_map_name,
    horizontal = horizontal,
    model_rsq = FALSE,
    max_feature = max_feature,
    cutoff = cutoff,
    title = title,
    xtitle = xtitle,
    ytitle = ytitle,
    rotation = rotation,
    label = label,
    decimal = decimal,
    save_name = save_name
  )
}

# Histogram of Shapley R^2 contributions (distribution)
vis$hist <- function(
  x,
  bins = 30,
  title = expression(paste("Distribution of Shapley ", R^2, " Contributions")),
  xtitle = expression(paste("Shapley ", R^2, " contribution")),
  ytitle = "Density",
  trim_nonfinite = TRUE,
  show_density = TRUE,
  density_adjust = 1,
  rotation = 0,
  main_color = "black",
  fill_color = "grey80",
  alpha = 0.85,
  save_name = NULL
) {
  x <- as.numeric(x)
  if (isTRUE(trim_nonfinite)) x <- x[is.finite(x)]
  if (length(x) == 0L) stop("No finite values to plot.")

  df <- data.frame(value = x)

  p <- ggplot(df, aes(x = value)) +
    geom_histogram(
      aes(y = after_stat(density)),
      bins = as.integer(bins),
      color = main_color,
      fill = fill_color,
      alpha = alpha
    ) +
    { if (isTRUE(show_density)) geom_density(adjust = density_adjust, linewidth = 0.9, color = main_color) else NULL } +
    labs(title = title, x = xtitle, y = ytitle) +
    theme_classic(base_size = 12) +
    theme(
      plot.title = element_text(face = "bold", hjust = 0),
      axis.title = element_text(face = "bold"),
      axis.text.x = element_text(angle = rotation, hjust = 1, vjust = 1),
      panel.grid.major.y = element_line(color = "grey85"),
      panel.grid.minor = element_blank()
    )

  if (!is.null(save_name)) {
    ggsave(filename = paste0(save_name, ".pdf"), plot = p, width = 7, height = 4.2)
  }

  print(p)
  invisible(p)
}

# Density-only plot of Shapley R^2 contributions
vis$density <- function(
  x,
  title = expression(paste("Density of Shapley ", R^2, " Contributions")),
  xtitle = expression(paste("Shapley ", R^2, " contribution")),
  ytitle = "Density",
  trim_nonfinite = TRUE,
  density_adjust = 1,
  rotation = 0,
  main_color = "black",
  save_name = NULL
) {
  x <- as.numeric(x)
  if (isTRUE(trim_nonfinite)) x <- x[is.finite(x)]
  if (length(x) == 0L) stop("No finite values to plot.")

  df <- data.frame(value = x)

  p <- ggplot(df, aes(x = value)) +
    geom_density(adjust = density_adjust, linewidth = 0.9, color = main_color) +
    labs(title = title, x = xtitle, y = ytitle) +
    theme_classic(base_size = 12) +
    theme(
      plot.title = element_text(face = "bold", hjust = 0),
      axis.title = element_text(face = "bold"),
      axis.text.x = element_text(angle = rotation, hjust = 1, vjust = 1),
      panel.grid.major.y = element_line(color = "grey85"),
      panel.grid.minor = element_blank()
    )

  if (!is.null(save_name)) {
    ggsave(filename = paste0(save_name, ".pdf"), plot = p, width = 7, height = 4.2)
  }

  print(p)
  invisible(p)
}


#' Plot Q-SHAP R-squared contributions
#'
#' Convenience wrapper that works for both a `qshap_rsq` object and a plain
#' numeric vector of contributions. Use this if you have a numeric vector and
#' still want to pass arguments like `color_map_name`.
#'
#' @param x A `qshap_rsq` object (recommended) or a numeric vector.
#' @param type Plot type; see `plot.qshap_rsq`. Use `"loss"` to launch the interactive explorer (requires a loss matrix).
#' @param ... Additional arguments passed to the underlying visualization
#'   function (e.g., `label`, `rotation`, `color_map_name`, `max_feature`).
#'
#' @return The ggplot2 plot object (invisibly)
#'
#' @examples
#' library(xgboost)
#' set.seed(42)
#' n <- 100
#' p <- 100
#' X <- matrix(rnorm(n * p), nrow = n, ncol = p)
#' y <- X[, 1] - X[, 2] + rnorm(n, sd = 0.2)
#' model <- xgboost(X, y, nrounds = 15L, max_depth = 2L, verbosity = 0L, nthreads = 1L)
#' explainer <- gazer(model)
#' phi_rsq <- rsq(explainer, X, y)
#' plot(phi_rsq)
#'
#' @keywords internal
plot_qshap <- function(x, type = c("rsq", "elbow", "cumu", "gcorr", "hist", "density", "loss"), ...) {
  # If x is a qshap_rsq object, reuse the S3 method
  if (inherits(x, "qshap_rsq")) {
    return(plot(x, type = type, ...))
  }

  # Otherwise treat x as a numeric vector and call vis functions directly
  type <- match.arg(type)

  # Interactive loss explorer (Shiny)
  # - If x is a qshap_rsq object with $loss, use x$loss
  # - If x is already a loss matrix/array (n x p), use it directly
  if (identical(type, "loss")) {
    if (inherits(x, "qshap_rsq") && is.list(x) && !is.null(x$loss)) {
      return(invisible(vis$loss(x$loss, ...)))
    }
    if (is.matrix(x) || is.array(x)) {
      return(invisible(vis$loss(x, ...)))
    }
    stop("type='loss' expects a qshap_rsq object with $loss, or a loss matrix/array (n x p). If you used qshap_rsq(..., local=TRUE), pass rsq_cons[[2]] (the loss matrix).")
  }

  rsq_values <- as.numeric(x)

  invisible(
    switch(type,
      rsq = vis$rsq(rsq_values, ...),
      elbow = vis$elbow(rsq_values, ...),
      cumu = vis$cumu(rsq_values, ...),
      gcorr = vis$gcorr(rsq_values, ...),
      hist = vis$hist(rsq_values, ...),
      density = vis$density(rsq_values, ...)
    )
  )
}

#' Calculating Feature-Specific R-Squared Values for Boosting Trees
#'
#' The qshap package computes feature-specific R-squared values using Shapley
#' decomposition of the total R-squared for boosting trees built in \pkg{xgboost} and \pkg{lightgbm}.
#' It supports parallel computing.
#' 
#'
#' @details
#' The package provides fast computation of feature importance through Shapley
#' values for tree ensemble models. Main functions include:
#'
#' \itemize{
#'   \item \code{gazer()}: Create a Q-SHAP explainer from a trained model
#'   \item \code{rsq()}: Calculate feature-specific R-squared values
#'   \item \code{loss()}: Calculate feature-specific loss contributions
#'   \item \code{plot()}: Visualize R-squared values
#' }
#'
#' The method uses polynomial-time complexity for Shapley value calculation and
#' includes built-in support for multi-core processing.
#'
#' @author
#' Steven He, Zhongli Jiang, Min Zhang, Dabao Zhang
#'
#' @references
#' Zhongli Jiang, Min Zhang, and Dabao Zhang. 2025. Fast 
#' calculation of feature contributions in boosting trees. In 
#' Proceedings of the Forty-First Conference on Uncertainty in Artificial Intelligence (UAI '25), Vol. 286. JMLR.org, Article 
#' 82, 1859–1875.
#'
#' @examples
#' library(xgboost)
#' set.seed(42)
#' n <- 100
#' p <- 100
#' X <- matrix(rnorm(n * p), nrow = n, ncol = p)
#' y <- X[, 1] - X[, 2] + rnorm(n, sd = 0.2)
#' model <- xgboost(X, y, nrounds = 15, max_depth = 2, verbose = 0)
#' explainer <- gazer(model)
#' phi_rsq <- rsq(explainer, X, y)
#'
#' @docType package
#' @name qshap-package
#' @keywords internal
"_PACKAGE"

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

compute_treeshap <- function(x, children_left, children_right, feature, threshold, value, n_node_samples) {
    .Call(`_qshap_compute_treeshap`, x, children_left, children_right, feature, threshold, value, n_node_samples)
}

T2 <- function(x, tree_summary, store_v_invc, store_z, parallel = TRUE) {
    .Call(`_qshap_T2`, x, tree_summary, store_v_invc, store_z, parallel)
}

loss_treeshap <- function(x, y, tree_summary, store_v_invc, store_z, T0_x, learning_rate = 1.0) {
    .Call(`_qshap_loss_treeshap`, x, y, tree_summary, store_v_invc, store_z, T0_x, learning_rate)
}

store_complex_v_invc <- function(d) {
    .Call(`_qshap_store_complex_v_invc`, d)
}

store_complex_root <- function(d) {
    .Call(`_qshap_store_complex_root`, d)
}

#' @importFrom xgboost xgb.model.dt.tree
#' @importFrom stats predict
NULL


# Loss implementation for xgboost model
#' @keywords internal
qshap_loss_xgboost <- function(explainer, x, y, y_mean_ori = NULL) {
  model <- explainer$model
  store_v_invc <- explainer$store_v_invc
  store_z <- explainer$store_z
  base_score <- explainer$base_score
  xgb_trees <- explainer$trees # This is a list of simple_tree objects
  tree_summaries <- explainer$tree_summaries
  if (is.null(tree_summaries)) {
    tree_summaries <- lapply(xgb_trees, summarize_tree)
  }
  
  num_tree <- length(xgb_trees)
  
  if (!is.matrix(x)) {
    x <- as.matrix(x)
  }
  n <- nrow(x)
  p <- ncol(x)
  loss <- matrix(0, nrow = n, ncol = p)
  pb <- if (interactive()) progress::progress_bar$new(
      format = "Progress [:bar] :current/:total (:percent)",
      total = num_tree,
      clear = FALSE,
      width = 60
    ) else NULL

  cum_pred <- rep(base_score, n)
  # XGBoost R uses 0-based indexing for iterationrange with exclusive end
  # c(0, 1) = tree 0 only, c(0, 2) = trees 0+1, etc.
  # Note: c(0, n) and c(1, n) are equivalent (both give trees 0 to n-1)
  # also enable slicing by tree index

  # XGBoost R uses 1-based indexing for model list !!!!
  # https://xgboost.readthedocs.io/en/stable/tutorials/slicing_model.html

  for (i in seq_len(num_tree)) { # i is the 1-based loop index; tree index is i-1 (0-based)
  
    if (!is.null(pb)) pb$tick()

    local_res <- y - cum_pred

    shap_tree_i <- stats::predict(model[i], x, predcontrib = TRUE)
    tree_pred <- rowSums(shap_tree_i) - base_score
    T0_x_tree <- shap_tree_i[, -ncol(shap_tree_i), drop = FALSE]
    summary_tree <- tree_summaries[[i]]
    
    # Call C++ loss_treeshap with per-tree SHAP values (T0_x_tree) and correct residuals (local_res)
    # The learning rate for individual XGBoost trees in this SHAP context is effectively 1.0,
    # as tree outputs are already scaled.
    current_tree_loss <- loss_treeshap(x, local_res, summary_tree, store_v_invc, store_z, T0_x_tree, 1.0)
    
    if (i == 1) {
      loss <- current_tree_loss
    } else {
      loss <- loss + current_tree_loss
    }

    cum_pred <- cum_pred + tree_pred
  }
  
  loss
}


# Formats an xgboost model into a list of simple_tree objects
# Becareful that this part is different from the Python version, we scale leaf weights here 
#' @keywords internal
xgb_formatter <- function(model_json, max_depth) {
  # model_json can be:
  #  (1) a parsed list from jsonlite::fromJSON(..., simplifyVector = FALSE), or
  #  (2) a filename to a JSON model file, or
  #  (3) an xgb.Booster (we'll save->read automatically)

  # case (3): booster
  if (inherits(model_json, "xgb.Booster")) {
    fn <- tempfile(fileext = ".json")
    xgboost::xgb.save(model_json, fn)
    model_json <- jsonlite::fromJSON(fn, simplifyVector = FALSE)
    unlink(fn)
  }

  # case (2): filename
  if (is.character(model_json) && length(model_json) == 1L && file.exists(model_json)) {
    model_json <- jsonlite::fromJSON(model_json, simplifyVector = FALSE)
  }

  # case (1): parsed json list
  if (!is.list(model_json)) stop("model_json must be xgb.Booster, a parsed JSON list, or a JSON filename.")

  # ---- robust tree path (xgboost JSON differs across builds) ----
  gb <- model_json$learner$gradient_booster
  if (is.null(gb)) stop("JSON missing learner$gradient_booster")

  # common locations:
  # A) gb$model$trees
  trees_data <- gb$model$trees

  # B) gb$gbtree_model$trees (seen in many versions)
  if (is.null(trees_data)) trees_data <- gb$gbtree_model$trees

  # C) gb$model$gbtree_model_param + gb$model$trees (rare combos)
  if (is.null(trees_data) && !is.null(gb$model) && !is.null(gb$model$trees)) trees_data <- gb$model$trees

  if (is.null(trees_data)) {
    stop("Could not find trees in JSON. Inspect with names(model_json$learner$gradient_booster).")
  }

  out <- vector("list", length(trees_data))

  for (i in seq_along(trees_data)) {
    tr <- trees_data[[i]]

    # base_weights in XGBoost JSON are already the final tree outputs (eta-scaled during training)
    # Do NOT scale again here - the values are ready to use as-is
    base_w <- as.numeric(unlist(tr$base_weights))

    out[[i]] <- simple_tree(
      children_left  = as.integer(unlist(tr$left_children)),
      children_right = as.integer(unlist(tr$right_children)),
      feature        = as.integer(unlist(tr$split_indices)),
      threshold      = as.numeric(unlist(tr$split_conditions)),
      max_depth      = as.integer(max_depth),
      n_node_samples = as.numeric(unlist(tr$sum_hessian)),
      value          = base_w,
      node_count     = as.integer(tr$tree_param$num_nodes)
    )
  }

  out
}

#' S3 Class Constructors and Methods for qshap
#'
#' This file contains formal S3 class definitions, constructors, validators,
#' and methods for the qshap package objects.
#'
#' @name qshap-classes
#' @keywords internal
NULL

# ============================================================================
# simple_tree class
# ============================================================================

#' Constructor for simple_tree class
#' 
#' Creates a simple_tree object with validation
#' 
#' @param children_left Integer vector of left child indices (-1 for leaf nodes)
#' @param children_right Integer vector of right child indices (-1 for leaf nodes)
#' @param feature Integer vector of feature indices used for splitting (-1 for leaf nodes)
#' @param threshold Numeric vector of threshold values for splits
#' @param max_depth Integer maximum depth of the tree
#' @param n_node_samples Integer vector of sample counts at each node
#' @param value Numeric vector of node values
#' @param node_count Integer total number of nodes in the tree
#' 
#' @return An object of class \code{simple_tree}
#' @keywords internal
new_simple_tree <- function(children_left,
                            children_right,
                            feature,
                            threshold,
                            max_depth,
                            n_node_samples,
                            value,
                            node_count) {
  structure(
    list(
      children_left   = as.integer(children_left),
      children_right  = as.integer(children_right),
      feature         = as.integer(feature),
      threshold       = as.numeric(threshold),
      max_depth       = as.integer(max_depth),
      n_node_samples  = as.integer(n_node_samples),
      value           = as.numeric(value),
      node_count      = as.integer(node_count)
    ),
    class = "simple_tree"
  )
}

#' Validator for simple_tree class
#' 
#' @param x A simple_tree object
#' @return The validated object (invisibly) or stops with an error
#' @keywords internal
validate_simple_tree <- function(x) {
  if (!inherits(x, "simple_tree")) {
    stop("Object must be of class 'simple_tree'", call. = FALSE)
  }
  
  # Check that all required fields are present
  required_fields <- c("children_left", "children_right", "feature", "threshold",
                       "max_depth", "n_node_samples", "value", "node_count")
  missing <- setdiff(required_fields, names(x))
  if (length(missing) > 0) {
    stop("Missing required fields: ", paste(missing, collapse = ", "), call. = FALSE)
  }
  
  # Check that vector lengths match node_count
  if (x$node_count > 0) {
    vec_lengths <- c(
      length(x$children_left),
      length(x$children_right),
      length(x$feature),
      length(x$threshold),
      length(x$n_node_samples),
      length(x$value)
    )
    if (!all(vec_lengths == x$node_count)) {
      stop("All node vectors must have length equal to node_count", call. = FALSE)
    }
  }
  
  # Check max_depth is positive
  if (x$max_depth < 0) {
    stop("max_depth must be non-negative", call. = FALSE)
  }
  
  invisible(x)
}

#' User-friendly constructor for simple_tree
#' 
#' @inheritParams new_simple_tree
#' @return A validated simple_tree object
#' @keywords internal
simple_tree <- function(children_left,
                        children_right,
                        feature,
                        threshold,
                        max_depth,
                        n_node_samples,
                        value,
                        node_count) {
  x <- new_simple_tree(
    children_left, children_right, feature, threshold,
    max_depth, n_node_samples, value, node_count
  )
  validate_simple_tree(x)
  x
}

#' Print method for simple_tree
#' 
#' @param x A simple_tree object
#' @param ... Additional arguments (currently unused)
#' @return The input \code{x} is returned invisibly. Called primarily for its
#'   side effect of printing a summary of the \code{simple_tree} object to the
#'   console.
#' @export
print.simple_tree <- function(x, ...) {
  cat("<simple_tree>\n")
  cat("  Nodes:", x$node_count, "\n")
  cat("  Max depth:", x$max_depth, "\n")
  cat("  Leaf nodes:", sum(x$children_left == -1), "\n")
  cat("  Split nodes:", sum(x$children_left != -1), "\n")
  invisible(x)
}

# ============================================================================
# tree_summary class
# ============================================================================

#' Constructor for tree_summary class
#' 
#' Creates a tree_summary object with validation
#' 
#' @param children_left Integer vector of left child indices
#' @param children_right Integer vector of right child indices
#' @param feature Integer vector of feature indices
#' @param feature_uniq Integer vector of unique feature indices used in tree
#' @param threshold Numeric vector of threshold values
#' @param max_depth Integer maximum depth
#' @param sample_weight Numeric vector of sample weights per node
#' @param init_prediction Numeric vector of initial predictions per node
#' @param node_count Integer total number of nodes
#' 
#' @return An object of class \code{tree_summary}
#' @keywords internal
new_tree_summary <- function(children_left,
                             children_right,
                             feature,
                             feature_uniq,
                             threshold,
                             max_depth,
                             sample_weight,
                             init_prediction,
                             node_count) {
  structure(
    list(
      children_left   = as.integer(children_left),
      children_right  = as.integer(children_right),
      feature         = as.integer(feature),
      feature_uniq    = as.integer(feature_uniq),
      threshold       = as.numeric(threshold),
      max_depth       = as.integer(max_depth),
      sample_weight   = as.numeric(sample_weight),
      init_prediction = as.numeric(init_prediction),
      node_count      = as.integer(node_count)
    ),
    class = "tree_summary"
  )
}

#' Validator for tree_summary class
#' 
#' @param x A tree_summary object
#' @return The validated object (invisibly) or stops with an error
#' @keywords internal
validate_tree_summary <- function(x) {
  if (!inherits(x, "tree_summary")) {
    stop("Object must be of class 'tree_summary'", call. = FALSE)
  }
  
  # Check required fields
  required_fields <- c("children_left", "children_right", "feature", "feature_uniq",
                       "threshold", "max_depth", "sample_weight", "init_prediction", "node_count")
  missing <- setdiff(required_fields, names(x))
  if (length(missing) > 0) {
    stop("Missing required fields: ", paste(missing, collapse = ", "), call. = FALSE)
  }
  
  # Check vector lengths
  if (x$node_count > 0) {
    vec_lengths <- c(
      length(x$children_left),
      length(x$children_right),
      length(x$feature),
      length(x$threshold),
      length(x$sample_weight),
      length(x$init_prediction)
    )
    if (!all(vec_lengths == x$node_count)) {
      stop("All node vectors must have length equal to node_count", call. = FALSE)
    }
  }
  
  invisible(x)
}

#' User-friendly constructor for tree_summary
#' 
#' @inheritParams new_tree_summary
#' @return A validated tree_summary object
#' @keywords internal
tree_summary <- function(children_left,
                         children_right,
                         feature,
                         feature_uniq,
                         threshold,
                         max_depth,
                         sample_weight,
                         init_prediction,
                         node_count) {
  x <- new_tree_summary(
    children_left, children_right, feature, feature_uniq,
    threshold, max_depth, sample_weight, init_prediction, node_count
  )
  validate_tree_summary(x)
  x
}

#' Print method for tree_summary
#' 
#' @param x A tree_summary object
#' @param ... Additional arguments (currently unused)
#' @return The input \code{x} is returned invisibly. Called primarily for its
#'   side effect of printing a summary of the \code{tree_summary} object to the
#'   console.
#' @export
print.tree_summary <- function(x, ...) {
  cat("<tree_summary>\n")
  cat("  Nodes:", x$node_count, "\n")
  cat("  Max depth:", x$max_depth, "\n")
  cat("  Unique features:", length(x$feature_uniq), "\n")
  cat("  Total init prediction:", round(sum(x$init_prediction), 4), "\n")
  invisible(x)
}

# ============================================================================
# qshap_tree_explainer class
# ============================================================================

#' Constructor for qshap_tree_explainer class
#' 
#' Creates a qshap_tree_explainer object
#' 
#' @param model The original tree model object
#' @param model_type Character string indicating model type ("xgboost" or "lightgbm")
#' @param max_depth Integer maximum tree depth
#' @param base_score Numeric base score (for XGBoost)
#' @param trees List of tree objects
#' @param store_v_invc Precomputed complex values for SHAP computation
#' @param store_z Precomputed root values for SHAP computation
#' 
#' @return An object of class \code{qshap_tree_explainer}
#' @keywords internal
new_qshap_tree_explainer <- function(model,
                                      model_type,
                                      max_depth,
                                      base_score = NULL,
                                      trees,
                                      store_v_invc,
                                      store_z) {
  obj <- list(
    model = model,
    model_type = as.character(model_type),
    max_depth = as.integer(max_depth),
    trees = trees,
    store_v_invc = store_v_invc,
    store_z = store_z
  )
  
  # Add base_score only for XGBoost
  if (!is.null(base_score)) {
    obj$base_score <- as.numeric(base_score)
  }
  
  # Set appropriate classes
  if (model_type == "xgboost") {
    class(obj) <- c("qshap_tree_explainer", "xgboost_explainer")
  } else if (model_type == "lightgbm") {
    class(obj) <- c("qshap_tree_explainer", "lightgbm_explainer")
  } else if (model_type == "catboost") {
    class(obj) <- c("qshap_tree_explainer", "catboost_explainer")
  } else {
    class(obj) <- "qshap_tree_explainer"
  }
  
  obj
}

#' Validator for qshap_tree_explainer
#' 
#' @param x A qshap_tree_explainer object
#' @return The validated object (invisibly) or stops with an error
#' @keywords internal
validate_qshap_tree_explainer <- function(x) {
  if (!inherits(x, "qshap_tree_explainer")) {
    stop("Object must be of class 'qshap_tree_explainer'", call. = FALSE)
  }
  
  # Check required fields
  required_fields <- c("model", "model_type", "max_depth", "trees", "store_v_invc", "store_z")
  missing <- setdiff(required_fields, names(x))
  if (length(missing) > 0) {
    stop("Missing required fields: ", paste(missing, collapse = ", "), call. = FALSE)
  }
  
  # Validate model_type
  if (!x$model_type %in% c("xgboost", "lightgbm", "catboost")) {
    stop("model_type must be 'xgboost', 'lightgbm', or 'catboost'", call. = FALSE)
  }
  
  # Check base_score for XGBoost and CatBoost
  if (x$model_type %in% c("xgboost", "catboost") && is.null(x$base_score)) {
    stop("base_score is required for XGBoost and CatBoost models", call. = FALSE)
  }
  
  # Check trees is a list
  if (!is.list(x$trees)) {
    stop("trees must be a list", call. = FALSE)
  }
  
  # Check max_depth is positive
  if (x$max_depth <= 0) {
    stop("max_depth must be positive", call. = FALSE)
  }
  
  invisible(x)
}

#' Print method for qshap_tree_explainer
#' 
#' @param x A qshap_tree_explainer object
#' @param ... Additional arguments (currently unused)
#' @return The input \code{x} is returned invisibly. Called primarily for its
#'   side effect of printing a summary of the \code{qshap_tree_explainer} object
#'   to the console.
#' @export
print.qshap_tree_explainer <- function(x, ...) {
  cat("<qshap_tree_explainer>\n")
  cat("  Model type:", x$model_type, "\n")
  cat("  Number of trees:", length(x$trees), "\n")
  cat("  Max depth:", x$max_depth, "\n")
  if (!is.null(x$base_score)) {
    cat("  Base score:", round(x$base_score, 4), "\n")
  }
  invisible(x)
}

#' Summary method for qshap_tree_explainer
#' 
#' Provides detailed summary information about the explainer
#' 
#' @param object A qshap_tree_explainer object
#' @param ... Additional arguments (currently unused)
#' @return The input \code{object} is returned invisibly. Called primarily for
#'   its side effect of printing a detailed summary of the
#'   \code{qshap_tree_explainer} object to the console.
#' @export
summary.qshap_tree_explainer <- function(object, ...) {
  cat("Q-SHAP Tree Explainer Summary\n")
  cat("=============================\n\n")
  
  cat("Model Information:\n")
  cat("  Type:", object$model_type, "\n")
  cat("  Number of trees:", length(object$trees), "\n")
  cat("  Maximum tree depth:", object$max_depth, "\n")
  
  if (!is.null(object$base_score)) {
    cat("  Base score:", round(object$base_score, 6), "\n")
  }
  
  cat("\nTree Structure Statistics:\n")
  if (length(object$trees) > 0) {
    # Get node counts from first few trees
    node_counts <- sapply(object$trees[1:min(5, length(object$trees))], 
                          function(t) t$node_count)
    cat("  Average nodes per tree (first 5):", round(mean(node_counts), 1), "\n")
    cat("  Node count range (first 5):", min(node_counts), "-", max(node_counts), "\n")
  }
  
  cat("\nComputation Resources:\n")
  cat("  Precomputed matrices initialized: Yes\n")
  cat("  Store dimension (v_invc):", 
      if(is.matrix(object$store_v_invc)) nrow(object$store_v_invc) else "N/A", "\n")
  
  invisible(object)
}

# ============================================================================
# qshap_result class
# ============================================================================

#' Constructor for qshap_result class
#' 
#' Creates a qshap_result object to store Q-SHAP R-squared results
#' 
#' @param rsq Numeric vector of feature-specific R-squared values
#' @param feature_names Character vector of feature names (optional)
#' @param total_rsq Numeric total R-squared (sum of feature-specific values)
#' @param n_samples Integer number of samples used
#' @param n_features Integer number of features
#' @param loss Optional loss matrix (n_samples x n_features)
#' 
#' @return An object of class \code{qshap_result}
#' @keywords internal
new_qshap_result <- function(rsq, 
                             feature_names = NULL,
                             total_rsq = NULL,
                             n_samples = NULL,
                             n_features = NULL,
                             loss = NULL) {
  if (is.null(total_rsq)) {
    total_rsq <- sum(rsq, na.rm = TRUE)
  }
  if (is.null(n_features)) {
    n_features <- length(rsq)
  }
  if (is.null(feature_names)) {
    feature_names <- paste0("Feature_", seq_along(rsq))
  }
  
  structure(
    list(
      rsq = as.numeric(rsq),
      feature_names = as.character(feature_names),
      total_rsq = as.numeric(total_rsq),
      n_samples = if (!is.null(n_samples)) as.integer(n_samples) else NULL,
      n_features = as.integer(n_features),
      loss = loss
    ),
    class = "qshap_result"
  )
}

#' Validator for qshap_result
#' 
#' @param x A qshap_result object
#' @return The validated object (invisibly) or stops with an error
#' @keywords internal
validate_qshap_result <- function(x) {
  if (!inherits(x, "qshap_result")) {
    stop("Object must be of class 'qshap_result'", call. = FALSE)
  }
  
  required_fields <- c("rsq", "feature_names", "total_rsq", "n_features")
  missing <- setdiff(required_fields, names(x))
  if (length(missing) > 0) {
    stop("Missing required fields: ", paste(missing, collapse = ", "), call. = FALSE)
  }
  
  # Check lengths match
  if (length(x$rsq) != length(x$feature_names)) {
    stop("Length of rsq must match length of feature_names", call. = FALSE)
  }
  
  if (length(x$rsq) != x$n_features) {
    stop("Length of rsq must equal n_features", call. = FALSE)
  }
  
  invisible(x)
}

#' User-friendly constructor for qshap_result
#' 
#' @inheritParams new_qshap_result
#' @return A validated qshap_result object
#' @export
qshap_result <- function(rsq, 
                         feature_names = NULL,
                         total_rsq = NULL,
                         n_samples = NULL,
                         n_features = NULL,
                         loss = NULL) {
  x <- new_qshap_result(rsq, feature_names, total_rsq, n_samples, n_features, loss)
  validate_qshap_result(x)
  x
}

#' Print method for qshap_result
#' 
#' @param x A qshap_result object
#' @param n Integer number of top features to display (default: 10)
#' @param ... Additional arguments (currently unused)
#' @return The input \code{x} is returned invisibly. Called primarily for its
#'   side effect of printing a summary of the \code{qshap_result} object to the
#'   console.
#' @export
print.qshap_result <- function(x, n = 10, ...) {
  cat("<qshap_result>\n")
  cat("  Total R^2:", round(x$total_rsq, 4), "\n")
  cat("  Number of features:", x$n_features, "\n")
  if (!is.null(x$n_samples)) {
    cat("  Number of samples:", x$n_samples, "\n")
  }
  
  cat("\nTop", min(n, x$n_features), "features by R^2:\n")
  
  # Create a data frame and sort by R^2
  df <- data.frame(
    Feature = x$feature_names,
    R_squared = x$rsq,
    stringsAsFactors = FALSE
  )
  df <- df[order(df$R_squared, decreasing = TRUE), ]
  df <- utils::head(df, n)
  
  # Print as a formatted table
  print(df, row.names = FALSE, digits = 4)
  
  if (x$n_features > n) {
    cat("... and", x$n_features - n, "more features\n")
  }
  
  invisible(x)
}

#' Summary method for qshap_result
#' 
#' @param object A qshap_result object
#' @param ... Additional arguments (currently unused)
#' @return The input \code{object} is returned invisibly. Called primarily for
#'   its side effect of printing a detailed summary of the \code{qshap_result}
#'   object to the console.
#' @export
summary.qshap_result <- function(object, ...) {
  cat("Q-SHAP Results Summary\n")
  cat("======================\n\n")
  
  cat("Overall Statistics:\n")
  cat("  Total R^2:", round(object$total_rsq, 6), "\n")
  cat("  Number of features:", object$n_features, "\n")
  if (!is.null(object$n_samples)) {
    cat("  Number of samples:", object$n_samples, "\n")
  }
  
  cat("\nR^2 Distribution:\n")
  cat("  Min:", round(min(object$rsq, na.rm = TRUE), 6), "\n")
  cat("  Q1:", round(stats::quantile(object$rsq, 0.25, na.rm = TRUE), 6), "\n")
  cat("  Median:", round(stats::median(object$rsq, na.rm = TRUE), 6), "\n")
  cat("  Mean:", round(mean(object$rsq, na.rm = TRUE), 6), "\n")
  cat("  Q3:", round(stats::quantile(object$rsq, 0.75, na.rm = TRUE), 6), "\n")
  cat("  Max:", round(max(object$rsq, na.rm = TRUE), 6), "\n")
  
  # Count significant features
  sig_features <- sum(object$rsq > 0.01, na.rm = TRUE)
  cat("\nSignificant Features (R^2 > 0.01):", sig_features, "\n")
  
  invisible(object)
}

#' Coercion method to data.frame for qshap_result
#' 
#' @param x A qshap_result object
#' @param row.names Not used
#' @param optional Not used
#' @param ... Additional arguments (currently unused)
#' @return A \code{data.frame} with columns \code{feature} (character) and
#'   \code{rsq} (numeric), sorted by \code{rsq} in decreasing order.
#' @export
as.data.frame.qshap_result <- function(x, row.names = NULL, optional = FALSE, ...) {
  df <- data.frame(
    feature = x$feature_names,
    rsq = x$rsq,
    stringsAsFactors = FALSE
  )
  
  # Sort by R^2 descending
  df <- df[order(df$rsq, decreasing = TRUE), ]
  rownames(df) <- NULL
  
  df
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{summary.qshap_tree_explainer}
\alias{summary.qshap_tree_explainer}
\title{Summary method for qshap_tree_explainer}
\usage{
\method{summary}{qshap_tree_explainer}(object, ...)
}
\arguments{
\item{object}{A qshap_tree_explainer object}

\item{...}{Additional arguments (currently unused)}
}
\value{
The input \code{object} is returned invisibly. Called primarily for
  its side effect of printing a detailed summary of the
  \code{qshap_tree_explainer} object to the console.
}
\description{
Provides detailed summary information about the explainer
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tree_explainer.R
\name{rsq}
\alias{rsq}
\title{Calculate Feature-Specific R-Squared Values}
\usage{
rsq(
  explainer,
  x,
  y,
  feature_names = NULL,
  local = FALSE,
  nsample = NULL,
  sd_out = TRUE,
  ci_out = TRUE,
  level = 0.95,
  nfrac = NULL,
  random_state = 42,
  ncore = 1L
)
}
\arguments{
\item{explainer}{A qshap_tree_explainer object created by \code{gazer()}}

\item{x}{Feature matrix or data frame with n samples and p features}

\item{y}{Response vector of length n}

\item{feature_names}{Character vector of feature names. If NULL, uses column names from x.}

\item{local}{Logical; if TRUE, returns both R-squared values and loss matrix}

\item{nsample}{Optional integer; number of samples to use (random subsample if less than nrow(x))}

\item{sd_out}{Logical; if TRUE, returns standard deviations of R-squared estimates}

\item{ci_out}{Logical; if TRUE, returns Wald-style confidence intervals for each feature's R-squared (normal approximation using sd_rsq)}

\item{level}{Confidence level for the intervals (default 0.95)}

\item{nfrac}{Optional numeric in (0,1); fraction of samples to use (alternative to nsample)}

\item{random_state}{Integer seed for reproducible sampling}

\item{ncore}{Number of cores for parallel processing. Use -1 for all available cores, 
or a positive integer. Default is 1 (no parallelization)}
}
\value{
A \code{qshap_result} object containing:
  \itemize{
    \item \code{rsq}: Numeric vector of feature-specific R² values
    \item \code{feature_names}: Character vector of feature names
    \item \code{total_rsq}: Total R² (sum of feature-specific values)
    \item \code{n_samples}: Number of samples
    \item \code{n_features}: Number of features
    \item \code{loss}: Loss matrix (if local=TRUE)
  }
}
\description{
Computes feature-specific R-squared values using Q-SHAP decomposition,
returning a \code{qshap_result} object with better formatting and additional metadata.
The \code{qshap_result} object includes feature names, total R², sample counts,
and provides enhanced \code{print()}, \code{summary()}, and \code{as.data.frame()}
methods for easier analysis.
}
\details{
This function provides a user-friendly interface for Q-SHAP R² computation:
\itemize{
  \item Automatically extracts feature names from the input data
  \item Returns a structured object with metadata
  \item Provides enhanced printing with top features displayed by default
  \item Includes a comprehensive \code{summary()} method
  \item Can be easily converted to a data frame with \code{as.data.frame()}
}
}
\examples{
library(xgboost)
set.seed(42)
n <- 100
p <- 100
X <- matrix(rnorm(n * p), nrow = n, ncol = p)
y <- X[, 1] - X[, 2] + rnorm(n, sd = 0.2)
model <- xgboost(X, y, nrounds = 15, max_depth = 2, verbose = 0)
explainer <- gazer(model)
result <- rsq(explainer, X, y)
print(result)

}
\seealso{
\code{\link{qshap_result}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{qshap_result}
\alias{qshap_result}
\title{User-friendly constructor for qshap_result}
\usage{
qshap_result(
  rsq,
  feature_names = NULL,
  total_rsq = NULL,
  n_samples = NULL,
  n_features = NULL,
  loss = NULL
)
}
\arguments{
\item{rsq}{Numeric vector of feature-specific R-squared values}

\item{feature_names}{Character vector of feature names (optional)}

\item{total_rsq}{Numeric total R-squared (sum of feature-specific values)}

\item{n_samples}{Integer number of samples used}

\item{n_features}{Integer number of features}

\item{loss}{Optional loss matrix (n_samples x n_features)}
}
\value{
A validated qshap_result object
}
\description{
User-friendly constructor for qshap_result
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{summary.qshap_result}
\alias{summary.qshap_result}
\title{Summary method for qshap_result}
\usage{
\method{summary}{qshap_result}(object, ...)
}
\arguments{
\item{object}{A qshap_result object}

\item{...}{Additional arguments (currently unused)}
}
\value{
The input \code{object} is returned invisibly. Called primarily for
  its side effect of printing a detailed summary of the \code{qshap_result}
  object to the console.
}
\description{
Summary method for qshap_result
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_module.R
\name{plot.qshap_rsq}
\alias{plot.qshap_rsq}
\title{Plot method for qshap_rsq objects}
\usage{
\method{plot}{qshap_rsq}(
  x,
  y = NULL,
  type = c("rsq", "elbow", "cumu", "gcorr", "hist", "density", "loss"),
  ...
)
}
\arguments{
\item{x}{A `qshap_rsq` object.}

\item{y}{Not used.}

\item{type}{Plot type: one of "rsq", "elbow", "cumu", "gcorr", "hist", "density", or "loss".}

\item{...}{Passed to the underlying visualization function.}
}
\value{
A ggplot2 object (invisibly).
}
\description{
This S3 method enables `plot(x, ...)` where `x` is a `qshap_rsq` object.
It dispatches to the visualization functions in `vis`.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{simple_tree}
\alias{simple_tree}
\title{User-friendly constructor for simple_tree}
\usage{
simple_tree(
  children_left,
  children_right,
  feature,
  threshold,
  max_depth,
  n_node_samples,
  value,
  node_count
)
}
\arguments{
\item{children_left}{Integer vector of left child indices (-1 for leaf nodes)}

\item{children_right}{Integer vector of right child indices (-1 for leaf nodes)}

\item{feature}{Integer vector of feature indices used for splitting (-1 for leaf nodes)}

\item{threshold}{Numeric vector of threshold values for splits}

\item{max_depth}{Integer maximum depth of the tree}

\item{n_node_samples}{Integer vector of sample counts at each node}

\item{value}{Numeric vector of node values}

\item{node_count}{Integer total number of nodes in the tree}
}
\value{
A validated simple_tree object
}
\description{
User-friendly constructor for simple_tree
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_module.R
\name{plot_qshap}
\alias{plot_qshap}
\title{Plot Q-SHAP R-squared contributions}
\usage{
plot_qshap(
  x,
  type = c("rsq", "elbow", "cumu", "gcorr", "hist", "density", "loss"),
  ...
)
}
\arguments{
\item{x}{A `qshap_rsq` object (recommended) or a numeric vector.}

\item{type}{Plot type; see `plot.qshap_rsq`. Use `"loss"` to launch the interactive explorer (requires a loss matrix).}

\item{...}{Additional arguments passed to the underlying visualization
function (e.g., `label`, `rotation`, `color_map_name`, `max_feature`).}
}
\value{
The ggplot2 plot object (invisibly)
}
\description{
Convenience wrapper that works for both a `qshap_rsq` object and a plain
numeric vector of contributions. Use this if you have a numeric vector and
still want to pass arguments like `color_map_name`.
}
\examples{
library(xgboost)
set.seed(42)
n <- 100
p <- 100
X <- matrix(rnorm(n * p), nrow = n, ncol = p)
y <- X[, 1] - X[, 2] + rnorm(n, sd = 0.2)
model <- xgboost(X, y, nrounds = 15L, max_depth = 2L, verbosity = 0L, nthreads = 1L)
explainer <- gazer(model)
phi_rsq <- rsq(explainer, X, y)
plot(phi_rsq)

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{print.simple_tree}
\alias{print.simple_tree}
\title{Print method for simple_tree}
\usage{
\method{print}{simple_tree}(x, ...)
}
\arguments{
\item{x}{A simple_tree object}

\item{...}{Additional arguments (currently unused)}
}
\value{
The input \code{x} is returned invisibly. Called primarily for its
  side effect of printing a summary of the \code{simple_tree} object to the
  console.
}
\description{
Print method for simple_tree
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{as.data.frame.qshap_result}
\alias{as.data.frame.qshap_result}
\title{Coercion method to data.frame for qshap_result}
\usage{
\method{as.data.frame}{qshap_result}(x, row.names = NULL, optional = FALSE, ...)
}
\arguments{
\item{x}{A qshap_result object}

\item{row.names}{Not used}

\item{optional}{Not used}

\item{...}{Additional arguments (currently unused)}
}
\value{
A \code{data.frame} with columns \code{feature} (character) and
  \code{rsq} (numeric), sorted by \code{rsq} in decreasing order.
}
\description{
Coercion method to data.frame for qshap_result
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tree_explainer.R
\name{loss}
\alias{loss}
\title{Alias for qshap_loss}
\usage{
loss(explainer, x, y, y_mean_ori = NULL)
}
\arguments{
\item{explainer}{A qshap_tree_explainer object created by \code{gazer()}}

\item{x}{Feature matrix or data frame}

\item{y}{Response vector}

\item{y_mean_ori}{Optional pre-computed mean of y (for efficiency)}
}
\value{
A matrix of loss contributions with dimensions (n_samples, n_features)
}
\description{
This is a convenience alias for \code{qshap_loss()} that provides a shorter
function name for calculating feature-specific loss contributions.
}
\examples{
library(xgboost)
set.seed(42)
n <- 100
p <- 100
X <- matrix(rnorm(n * p), nrow = n, ncol = p)
y <- X[, 1] - X[, 2] + rnorm(n, sd = 0.2)
model <- xgboost(X, y, nrounds = 15, max_depth = 2, verbose = 0)
explainer <- gazer(model)
loss_matrix <- loss(explainer, X, y)
dim(loss_matrix)

}
\seealso{
\code{\link{qshap_loss}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{new_tree_summary}
\alias{new_tree_summary}
\title{Constructor for tree_summary class}
\usage{
new_tree_summary(
  children_left,
  children_right,
  feature,
  feature_uniq,
  threshold,
  max_depth,
  sample_weight,
  init_prediction,
  node_count
)
}
\arguments{
\item{children_left}{Integer vector of left child indices}

\item{children_right}{Integer vector of right child indices}

\item{feature}{Integer vector of feature indices}

\item{feature_uniq}{Integer vector of unique feature indices used in tree}

\item{threshold}{Numeric vector of threshold values}

\item{max_depth}{Integer maximum depth}

\item{sample_weight}{Numeric vector of sample weights per node}

\item{init_prediction}{Numeric vector of initial predictions per node}

\item{node_count}{Integer total number of nodes}
}
\value{
An object of class \code{tree_summary}
}
\description{
Creates a tree_summary object with validation
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{print.tree_summary}
\alias{print.tree_summary}
\title{Print method for tree_summary}
\usage{
\method{print}{tree_summary}(x, ...)
}
\arguments{
\item{x}{A tree_summary object}

\item{...}{Additional arguments (currently unused)}
}
\value{
The input \code{x} is returned invisibly. Called primarily for its
  side effect of printing a summary of the \code{tree_summary} object to the
  console.
}
\description{
Print method for tree_summary
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{validate_tree_summary}
\alias{validate_tree_summary}
\title{Validator for tree_summary class}
\usage{
validate_tree_summary(x)
}
\arguments{
\item{x}{A tree_summary object}
}
\value{
The validated object (invisibly) or stops with an error
}
\description{
Validator for tree_summary class
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tree_explainer.R
\name{qshap_rsq}
\alias{qshap_rsq}
\title{Calculate Feature-Specific R-Squared Values}
\usage{
qshap_rsq(
  explainer,
  x,
  y,
  local = FALSE,
  nsample = NULL,
  sd_out = TRUE,
  ci_out = TRUE,
  level = 0.95,
  nfrac = NULL,
  random_state = 42,
  ncore = 1L
)
}
\arguments{
\item{explainer}{A qshap_tree_explainer object created by \code{gazer()}}

\item{x}{Feature matrix or data frame with n samples and p features}

\item{y}{Response vector of length n}

\item{local}{Logical; if TRUE, returns both R-squared values and loss matrix}

\item{nsample}{Optional integer; number of samples to use (random subsample if less than nrow(x))}

\item{sd_out}{Logical; if TRUE, returns standard deviations of R-squared estimates}

\item{ci_out}{Logical; if TRUE, returns Wald-style confidence intervals for each feature's R-squared (normal approximation using sd_rsq)}

\item{level}{Confidence level for the intervals (default 0.95)}

\item{nfrac}{Optional numeric in (0,1); fraction of samples to use (alternative to nsample)}

\item{random_state}{Integer seed for reproducible sampling}

\item{ncore}{Number of cores for parallel processing. Use -1 for all available cores, 
or a positive integer. Default is 1 (no parallelization)}
}
\value{
If \code{local=FALSE} (default), returns a numeric vector of length p 
  containing feature-specific R-squared values. If \code{local=TRUE}, returns 
  a list with components \code{rsq} (the R-squared vector) and \code{loss} 
  (an n x p matrix of loss contributions). When \code{ci_out=TRUE}, the returned list
  also contains \code{ci_lower} and \code{ci_upper} vectors representing Wald-style confidence intervals.
}
\description{
Computes feature-specific R-squared values using Q-SHAP decomposition.
Supports parallel processing and sampling for large datasets.
}
\examples{
library(xgboost)
set.seed(42)
n <- 100
p <- 100
X <- matrix(rnorm(n * p), nrow = n, ncol = p)
y <- X[, 1] - X[, 2] + rnorm(n, sd = 0.2)
model <- xgboost(X, y, nrounds = 15, max_depth = 2, verbose = 0)
explainer <- gazer(model)
phi_rsq <- qshap(explainer, X, y)
print(phi_rsq)

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tree_explainer.R
\name{gazer}
\alias{gazer}
\title{Create a QSHAP Tree Explainer}
\usage{
gazer(model, max_depth = NULL, base_score = NULL, ...)
}
\arguments{
\item{model}{A model object of class \code{xgboost} or \code{xgb.Booster} from \pkg{xgboost}, or class \code{lgb.Booster} from \pkg{lightgbm}}

\item{max_depth}{Maximum depth of trees, extracted from \code{model} by default.}

\item{base_score}{Base score for predictions, extracted from \code{model} by default.}

\item{...}{Additional arguments, for future use}
}
\value{
A class of \code{qshap_tree_explainer} object containing the model information and
  preprocessed tree structures for fast Shapley value computation
}
\description{
Creates an explainer object for computing feature-specific Shapley values
from a trained tree ensemble model. Supports XGBoost, LightGBM, and
CatBoost models.
}
\examples{
library(xgboost)
set.seed(42)
n <- 100
p <- 100
X <- matrix(rnorm(n * p), nrow = n, ncol = p)
y <- X[, 1] - X[, 2] + rnorm(n, sd = 0.2)
model <- xgboost(X, y, nrounds = 15, max_depth = 2, verbose = 0)
explainer <- gazer(model)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_module.R
\docType{data}
\name{vis}
\alias{vis}
\title{Visualization Module for Q-SHAP Results}
\format{
An environment with visualization functions:
\describe{
  \item{rsq}{Bar plot of feature-specific R-squared values}
  \item{elbow}{Elbow plot showing top contributing features}
  \item{cumu}{Cumulative explained variance plot}
  \item{gcorr}{Generalized correlation plot (square root of R-squared)}
  \item{hist}{Histogram of feature-specific R-squared contributions}
  \item{density}{Density plot of feature-specific R-squared contributions}
  \item{loss}{Interactive loss explorer (requires shiny)}
}
}
\usage{
vis
}
\description{
An environment containing visualization functions for Q-SHAP results.
Access functions using \code{vis$rsq()}, \code{vis$elbow()}, etc.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tree_explainer.R
\name{qshap_loss}
\alias{qshap_loss}
\title{Calculate Q-SHAP Loss Contributions}
\usage{
qshap_loss(explainer, x, y, y_mean_ori = NULL)
}
\arguments{
\item{explainer}{A qshap_tree_explainer object created by \code{gazer()}}

\item{x}{Feature matrix or data frame}

\item{y}{Response vector}

\item{y_mean_ori}{Optional pre-computed mean of y (for efficiency)}
}
\value{
A matrix of loss contributions with dimensions (n_samples, n_features)
}
\description{
Computes the feature-specific loss contributions using Q-SHAP decomposition.
This is an internal function typically called by \code{rsq()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{print.qshap_result}
\alias{print.qshap_result}
\title{Print method for qshap_result}
\usage{
\method{print}{qshap_result}(x, n = 10, ...)
}
\arguments{
\item{x}{A qshap_result object}

\item{n}{Integer number of top features to display (default: 10)}

\item{...}{Additional arguments (currently unused)}
}
\value{
The input \code{x} is returned invisibly. Called primarily for its
  side effect of printing a summary of the \code{qshap_result} object to the
  console.
}
\description{
Print method for qshap_result
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{qshap-classes}
\alias{qshap-classes}
\title{S3 Class Constructors and Methods for qshap}
\description{
This file contains formal S3 class definitions, constructors, validators,
and methods for the qshap package objects.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{new_qshap_tree_explainer}
\alias{new_qshap_tree_explainer}
\title{Constructor for qshap_tree_explainer class}
\usage{
new_qshap_tree_explainer(
  model,
  model_type,
  max_depth,
  base_score = NULL,
  trees,
  store_v_invc,
  store_z
)
}
\arguments{
\item{model}{The original tree model object}

\item{model_type}{Character string indicating model type ("xgboost" or "lightgbm")}

\item{max_depth}{Integer maximum tree depth}

\item{base_score}{Numeric base score (for XGBoost)}

\item{trees}{List of tree objects}

\item{store_v_invc}{Precomputed complex values for SHAP computation}

\item{store_z}{Precomputed root values for SHAP computation}
}
\value{
An object of class \code{qshap_tree_explainer}
}
\description{
Creates a qshap_tree_explainer object
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qshap-package.R
\docType{package}
\name{qshap-package}
\alias{qshap-package}
\title{Calculating Feature-Specific R-Squared Values for Boosting Trees}
\description{
The qshap package computes feature-specific R-squared values using Shapley
decomposition of the total R-squared for boosting trees built in \pkg{xgboost} and \pkg{lightgbm}.
It supports parallel computing.
}
\details{
The package provides fast computation of feature importance through Shapley
values for tree ensemble models. Main functions include:

\itemize{
  \item \code{gazer()}: Create a Q-SHAP explainer from a trained model
  \item \code{rsq()}: Calculate feature-specific R-squared values
  \item \code{loss()}: Calculate feature-specific loss contributions
  \item \code{plot()}: Visualize R-squared values
}

The method uses polynomial-time complexity for Shapley value calculation and
includes built-in support for multi-core processing.
}
\examples{
library(xgboost)
set.seed(42)
n <- 100
p <- 100
X <- matrix(rnorm(n * p), nrow = n, ncol = p)
y <- X[, 1] - X[, 2] + rnorm(n, sd = 0.2)
model <- xgboost(X, y, nrounds = 15, max_depth = 2, verbose = 0)
explainer <- gazer(model)
phi_rsq <- rsq(explainer, X, y)

}
\references{
Zhongli Jiang, Min Zhang, and Dabao Zhang. 2025. Fast 
calculation of feature contributions in boosting trees. In 
Proceedings of the Forty-First Conference on Uncertainty in Artificial Intelligence (UAI '25), Vol. 286. JMLR.org, Article 
82, 1859–1875.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/catstats/Q-SHAP_R}
  \item Report bugs at \url{https://github.com/catstats/Q-SHAP_R/issues}
}

}
\author{
Steven He, Zhongli Jiang, Min Zhang, Dabao Zhang
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vis_module.R
\name{summary.qshap_rsq}
\alias{summary.qshap_rsq}
\title{Summary method for qshap_rsq objects}
\usage{
\method{summary}{qshap_rsq}(object, n = 10, ...)
}
\arguments{
\item{object}{A \code{qshap_rsq} object}

\item{n}{Integer number of top features to display (default: 10)}

\item{...}{Additional arguments (currently unused)}
}
\value{
The input \code{object} is returned invisibly. Called primarily for
  its side effect of printing a summary of the \code{qshap_rsq} object to
  the console.
}
\description{
Provides a summary of the qshap_rsq object, showing the top features by R-squared contribution
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classes.R
\name{tree_summary}
\alias{tree_summary}
\title{User-friendly constructor for tree_summary}
\usage{
tree_summary(
  children_left,
  children_right,
  feature,
  feature_uniq,
  threshold,
  max_depth,
  sample_weight,
  init_prediction,
  node_count
)
}
\arguments{
\item{children_left}{Integer vector of left child indices}

\item{children_right}{Integer vector of right child indices}

\item{feature}{Integer vector of feature indices}

\item{feature_uniq}{Integer vector of unique feature indices used in tree}

\item{threshold}{Numeric vector of threshold values}

\item{max_depth}{Integer maximum depth}

\item{sample_weight}{Numeric vector of sample weights per node}

\item{init_prediction}{Numeric vector of initial predictions per node}

\item{node_count}{Integer total number of nodes}
}
\value{
A validated tree_summary object
}
\description{
User-friendly constructor for tree_summary
}
\keyword{internal}
